C     $Source: /storm/rc1/cvsroot/rc/rrtm_sw/src/cldprop.f,v $
C     $Author: jdelamer $
C     $Revision: 2.7 $
C     $Date: 2004/04/15 18:42:10 $
      SUBROUTINE CLDPROP(ICLDATM)

C
C  --------------------------------------------------------------------------
C |                                                                          |
C |  Copyright 2002, 2003, Atmospheric & Environmental Research, Inc. (AER). |
C |  This software may be used, copied, or redistributed as long as it is    |
C |  not sold and this copyright notice is reproduced on each copy made.     |
C |  This model is provided as is without any express or implied warranties. |
C |                       (http://www.rtweb.aer.com/)                        |
C |                                                                          |
C  --------------------------------------------------------------------------

C     PURPOSE:  COMPUTE THE CLOUD OPTICAL DEPTH(S) FOR EACH CLOUDY
C               LAYER.  NOTE:  ONLY INFLAG = 0 AND INFLAG=2/LIQFLAG=1,
C               ICEFLAG=2,3
C               (HU & STAMNES, Q. FU) ARE IMPLEMENTED.

      INCLUDE 'param.f'

      COMMON /CONTROL/   IAER, NSTR, IOUT, ISTART, IEND, ICLD 
      COMMON /PROFILE/   NLAYERS,PAVEL(MXLAY),TAVEL(MXLAY),
     &                   PZ(0:MXLAY),TZ(0:MXLAY)
      COMMON /CLOUDIN/   INFLAG,CLDDAT1(MXLAY),CLDDAT2(MXLAY),
     &                   ICEFLAG,LIQFLAG,CLDDAT3(MXLAY),
     &                   CLDDAT4(MXLAY),CLDDATMOM(0:16,MXLAY)

      COMMON /CLOUDDAT/  NCBANDS,CLDFRAC(MXLAY),
     &     TAUCLOUD(MXLAY,NBANDS),SSACLOUD(MXLAY,NBANDS),
     &     XMOM(0:16,MXLAY,NBANDS),TAUCLDORIG(MXLAY,NBANDS)

      COMMON /CLDOPTPROPS/ EXTLIQ1(58,NBANDS),
     &     SSALIQ1(58,NBANDS), ASYLIQ1(58,NBANDS),
     &     EXTICE2(43,NBANDS), SSAICE2(43,NBANDS),
     &     ASYICE2(43,NBANDS), 
     &     EXTICE3(46,NBANDS), SSAICE3(46,NBANDS),
     &     ASYICE3(46,NBANDS), FDLICE3(46,NBANDS)

      COMMON /CVRCLD/    HNAMCLD,HVRCLD

      CHARACTER*18       HNAMCLD,HVRCLD

      DIMENSION EXTCOLIQ(NBANDS),SSACOLIQ(NBANDS),GLIQ(NBANDS)
      DIMENSION EXTCOICE(NBANDS),SSACOICE(NBANDS),GICE(NBANDS)
      DIMENSION FORWLIQ(NBANDS),FORWICE(NBANDS)
      DIMENSION FDELTA(NBANDS)
      DIMENSION TAUTOT(NBANDS)

      DATA EPS /1.E-6/


C     Explanation of the method for each value of INFLAG.  A value of
C     0 for INFLAG do not distingish being liquid and ice clouds.
C     INFLAG = 2 does distinguish between liquid and ice clouds, and
C     requires further user input to specify the method to be used to 
C     compute the aborption due to each.
C     INFLAG = 0:  For each cloudy layer, the cloud fraction, the cloud optical
C                   depth, the cloud single-scattering albedo, and the
C                   moments of the phase function (0:NSTREAM).  Note
C                   that these values are delta-m scaled within this
C                   subroutine.
C     INFLAG = 2:  For each cloudy layer, the cloud fraction, cloud 
C                  water path (g/m2), and cloud ice fraction are input.
C       ICEFLAG = 2:  The ice effective radius (microns) is input and the
C                     optical properties due to ice clouds are computed from
C                     the optical properties stored in the RT code,
C                     STREAMER v3.0 (Reference: Key. J., Streamer 
C                     User's Guide, Cooperative Institute for
C                     Meteorological Satellite Studies, 2001, 96 pp.).
C                     Valid range of values for re are between 5.0 and
C                     131.0 micron.
C       ICEFLAG = 3: The ice generalized effective size (dge) is input
C                    and the optical depths, single-scattering albedo,
C                    and phase function moments are calculated as in
C                    Q. Fu, J. Climate, (1996). Q. Fu provided high resolution
C                    tables which were appropriately averaged for the
C                    bands in RRTM_SW.  Linear interpolation is used to
C                    get the coefficients from the stored tables.
C                    Valid range of values for dge are between 5.0 and
C                    140.0 micron.
C       LIQFLAG = 1:  The water droplet effective radius (microns) is input 
C                     and the optical depths due to water clouds are computed 
C                     as in Hu and Stamnes, J., Clim., 6, 728-742, (1993).
C                     The values for absorption coefficients appropriate for
C                     the spectral bands in RRTM have been obtained for a 
C                     range of effective radii by an averaging procedure 
C                     based on the work of J. Pinto (private communication).
C                     Linear interpolation is used to get the absorption 
C                     coefficients for the input effective radius.
C  

      HVRCLD = '$Revision: 2.7 $'

      ICLDATM = 0
      NCBANDS = 29
c Initialize cloud property variable
      TAUCLOUD(:,:) = 0.0
      SSACLOUD(:,:) = 0.0
      XMOM(:,:,:) = 0.0
      TAUTOT(:) = 0.0

c Open OUT_CLD_RRTM to output the cloud optical properties
      ICD = 25
      OPEN (ICD,FILE='OUT_CLD_RRTM',FORM='FORMATTED')
      WRITE(ICD,8900)
      NPRELAY = 0
         
      DO 3000 LAY = 1, NLAYERS
         IF (CLDFRAC(LAY) .GE. EPS) THEN
            ICLDATM = 1

C     Ice clouds and water clouds combined.
            IF (INFLAG .EQ. 0) THEN
               TAUCLDORIG_A = CLDDAT1(LAY)
               FFP = CLDDATMOM(NSTR,LAY)
               FFP1 = 1.0 - FFP
               FFPSSA = 1.0 - FFP * CLDDAT2(LAY)
               SSACLOUD_A = FFP1*CLDDAT2(LAY)/FFPSSA
               TAUCLOUD_A = FFPSSA*TAUCLDORIG_A
               DO 1010 IB = 16,NBANDS
                  TAUCLDORIG(LAY,IB) = TAUCLDORIG_A
                  SSACLOUD(LAY,IB) = SSACLOUD_A
                  TAUCLOUD(LAY,IB) = TAUCLOUD_A
                  DO 1000 ISTR = 0,NSTR
                     XMOM(ISTR,LAY,IB) = (CLDDATMOM(ISTR,LAY) - FFP)/
     &                    (FFP1)
 1000             CONTINUE
 1010          CONTINUE
C     Separate treatement of ice clouds and water clouds.
            ELSEIF(INFLAG .EQ. 2) THEN
               CWP = CLDDAT1(LAY)
               FICE = CLDDAT2(LAY)
               RADICE = CLDDAT3(LAY)
               
C     Calculation of absorption coefficients due to ice clouds.
               IF (FICE .EQ. 0.0) THEN
                  NCBANDS = 29
                  DO 2000 IB = 16, NCBANDS
                     EXTCOICE(IB) = 0.0
                     SSACOICE(IB) = 0.0
                     GICE(IB) = 0.0
                     FORWICE(IB) = 0.0
 2000             CONTINUE
               ELSEIF (ICEFLAG .EQ. 2) THEN
                  IF (RADICE .LT. 5.0 .OR. RADICE .GT. 131.)
     &                 STOP 'ICE RADIUS OUT OF BOUNDS'
                  NCBANDS = 29
                  FACTOR = (RADICE - 2.)/3.
                  INDEX = INT(FACTOR)
                  IF (INDEX .EQ. 43) INDEX = 42
                  FINT = FACTOR - FLOAT(INDEX)
                  DO 2200 IB = 16, NCBANDS
                     EXTCOICE(IB) = FICE * 
     &                    (EXTICE2(INDEX,IB) + FINT *
     &                    (EXTICE2(INDEX+1,IB) - 
     &                    EXTICE2(INDEX,IB)))
                     SSACOICE(IB) = SSAICE2(INDEX,IB) + FINT *
     &                    (SSAICE2(INDEX+1,IB) - 
     &                    SSAICE2(INDEX,IB))
                     GICE(IB) = ASYICE2(INDEX,IB) + FINT *
     &                    (ASYICE2(INDEX+1,IB) - 
     &                    ASYICE2(INDEX,IB))
                     FORWICE(IB) = GICE(IB)**NSTR
 2200             CONTINUE
               ELSEIF (ICEFLAG .EQ. 3) THEN
                  IF (RADICE .LT. 5.0 .OR. RADICE .GT. 140.0) STOP
     &                 'ICE EFFECTIVE SIZE OUT OF BOUNDS'
                  NCBANDS = 29
                  FACTOR = (RADICE - 2.)/3.
                  INDEX = INT(FACTOR)
                  IF (INDEX .EQ. 46) INDEX = 45
                  FINT = FACTOR - FLOAT(INDEX)
                  DO 2300 IB=16,NCBANDS
                     EXTCOICE(IB) = FICE * 
     &                    (EXTICE3(INDEX,IB) + FINT *
     &                    (EXTICE3(INDEX+1,IB) - 
     &                    EXTICE3(INDEX,IB)))
                     SSACOICE(IB) = SSAICE3(INDEX,IB) + 
     &                    FINT * (SSAICE3(INDEX+1,IB) - 
     &                    SSAICE3(INDEX,IB))
                     GICE(IB) = ASYICE3(INDEX,IB) + FINT *
     &                    (ASYICE3(INDEX+1,IB) - 
     &                    ASYICE3(INDEX,IB))
                     FDELTA(IB) = FDLICE3(INDEX,IB) + 
     &                    FINT * (FDLICE3(INDEX+1,IB) - 
     &                    FDLICE3(INDEX,IB))
                     if (fdelta(ib) .lt. 0.0) STOP
     &                    'FDELTA LESS THAN 0.0'
                     if (fdelta(ib) .gt. 1.0) STOP
     &                    'FDELTA GT THAN 1.0'                     
                     FORWICE(IB) = FDELTA(IB) + 0.5 / SSACOICE(IB)
c     See Fu 1996 p. 2067 
                     IF (FORWICE(IB) .GT. GICE(IB)) 
     &                    FORWICE(IB) = GICE(IB)
c     Check to ensure all calculated quantities are within physical limits.
                     if (extcoice(ib) .lt. 0.0) STOP
     &                    'ICE EXTINCTION LESS THAN 0.0'
                     if (ssacoice(ib) .gt. 1.0) STOP
     &                    'ICE SSA GRTR THAN 1.0'
                     if (ssacoice(ib) .lt. 0.0) STOP
     &                    'ICE SSA LESS THAN 0.0'
                     if (gice(ib) .gt. 1.0) STOP
     &                    'ICE ASYM GRTR THAN 1.0'
                     if (gice(ib) .lt. 0.0) STOP
     &                    'ICE ASYM LESS THAN 0.0'
 2300             CONTINUE
               ENDIF
               
C     Calculation of absorption coefficients due to water clouds.
               FLIQ = 1. - FICE
               IF (FLIQ .EQ. 0.0) THEN
                  NCBANDS = 29
                  DO 2350 IB = 16, NCBANDS
                     EXTCOLIQ(IB) = 0.0
                     SSACOLIQ(IB) = 0.0
                     GLIQ(IB) = 0.0
                     FORWLIQ(IB) = 0.0
 2350             CONTINUE
               ELSEIF (LIQFLAG .EQ. 1) THEN
                  RADLIQ = CLDDAT4(LAY)
                  IF (RADLIQ .LT. 1.5 .OR. RADLIQ .GT. 60.) STOP
     &                 'LIQUID EFFECTIVE RADIUS OUT OF BOUNDS'
                  INDEX = RADLIQ - 1.5
                  IF (INDEX .EQ. 0) INDEX = 1
                  IF (INDEX .EQ. 58) INDEX = 57
                  FINT = RADLIQ - 1.5 - INDEX
                  NCBANDS = 29
                  DO 2400 IB = 16, NCBANDS
                     EXTCOLIQ(IB) = FLIQ * (EXTLIQ1(INDEX,IB) + FINT *
     &                    (EXTLIQ1(INDEX+1,IB) - (EXTLIQ1(INDEX,IB))))
                     SSACOLIQ(IB) = SSALIQ1(INDEX,IB) + FINT *
     &                    (SSALIQ1(INDEX+1,IB) - SSALIQ1(INDEX,IB))
                     GLIQ(IB) = ASYLIQ1(INDEX,IB) + FINT *
     &                    (ASYLIQ1(INDEX+1,IB) - ASYLIQ1(INDEX,IB))
                     FORWLIQ(IB) = GLIQ(IB)**NSTR
c     Check to ensure all calculated quantities are within physical limits.
                     if (extcoliq(ib) .lt. 0.0) STOP
     &                    'LIQUID EXTINCTION LESS THAN 0.0'
                     if (ssacoliq(ib) .gt. 1.0) STOP
     &                    'LIQUID SSA GRTR THAN 1.0'
                     if (ssacoliq(ib) .lt. 0.0) STOP
     &                    'LIQUID SSA LESS THAN 0.0'
                     if (gliq(ib) .gt. 1.0) STOP
     &                    'LIQUID ASYM GRTR THAN 1.0'
                     if (gliq(ib) .lt. 0.0) STOP
     &                    'LIQUID ASYM LESS THAN 0.0'
 2400             CONTINUE
               ENDIF
               
C     KEEP RUNNING TOTAL OF OPTICAL DEPTH OF EACH CLOUD.  WHEN
C     A CLEAR LAYER SEPARATES CLOUDY LAYERS, RESET COUNTER
               
               IF (LAY .NE. NPRELAY+1) TAUTOT(:) = 0.0
               DO 2800 IB = 16, NCBANDS
                  TAULIQORIG = CWP * EXTCOLIQ(IB)
                  TAUICEORIG = CWP * EXTCOICE(IB)
                  TAUCLDORIG(LAY,IB) = TAULIQORIG + TAUICEORIG
                  SSALIQ = SSACOLIQ(IB) * (1. - FORWLIQ(IB)) /
     &                 (1. - FORWLIQ(IB) * SSACOLIQ(IB))
                  TAULIQ =  (1. - FORWLIQ(IB) * SSACOLIQ(IB)) *
     &                 TAULIQORIG
                  SSAICE = SSACOICE(IB) * (1. - FORWICE(IB)) /
     &                 (1. - FORWICE(IB) * SSACOICE(IB))
                  TAUICE =  (1. - FORWICE(IB) * SSACOICE(IB)) *
     &                 TAUICEORIG
                  SCATLIQ = SSALIQ * TAULIQ
                  SCATICE = SSAICE * TAUICE
                  
                  TAUCLOUD(LAY,IB) = TAULIQ + TAUICE
                  SSACLOUD(LAY,IB) = (SCATLIQ + SCATICE) / 
     &                 TAUCLOUD(LAY,IB)
                  TAUTOT(IB) = TAUTOT(IB)+TAUCLDORIG(LAY,IB)
                  WRITE(ICD,9000) LAY,IB,WAVENUM1(IB),WAVENUM2(IB),
     &                 TAUICEORIG,TAULIQORIG,
     &                 TAUCLDORIG(LAY,IB),
     &                 TAUTOT(IB),SSACOICE(IB),
     &                 SSACOLIQ(IB),
     &                 GICE(IB),GLIQ(IB)
                  XMOM(0,LAY,IB) = 1.0
                  IF (ICEFLAG .EQ. 3) THEN
c     In accordance with the 1996 Fu paper, equation A.3, 
c     the moments for ice were calculated depending on whether using spheres
c     or hexagonal ice crystals.
                     DO 2750 ISTR = 1, NSTR
                        XMOM(ISTR,LAY,IB) = 
     &                       (1.0/(scatliq+scatice))*
     &                       (SCATLIQ*(GLIQ(IB)**ISTR - 
     &                       FORWLIQ(IB)) / (1. - FORWLIQ(IB))
     &                       + SCATICE *((gice(ib)-forwice(ib))/
     &                       (1.0-forwice(ib)))**ISTR)
 2750                CONTINUE
                     
c     This code is the standard method for delta-m scaling. 
                     
                  ELSE
                     DO 2775 ISTR = 1, NSTR
                        XMOM(ISTR,LAY,IB) = (SCATLIQ * 
     &                       (GLIQ(IB)**ISTR - FORWLIQ(IB)) /
     &                       (1. - FORWLIQ(IB))
     &                       + SCATICE *
     &                       (GICE(IB)**ISTR - FORWICE(IB)) / 
     &                       (1. - FORWICE(IB)))/(SCATLIQ + 
     &                       SCATICE)
 2775                CONTINUE
                  ENDIF
 2800          CONTINUE
               
            ENDIF
            NPRELAY = LAY
         ENDIF
         
 3000 CONTINUE
      
      CLOSE(ICD)
 8900 FORMAT(2X,'LAY',1X,'BND ',1X,'WVN1    ','WVN2    ',
     &     'TAUICE       ','TAULIQ       ',
     &     'TAUTOT       ','CUMTAU       ','SSAICE       ',
     &     'SSALIQ       ',
     &     'GICE         ','GLIQ         ' )
 9000 FORMAT(2X,   I3,1X,I3,1X,2(F7.1,1x),8(E12.5,1X))
      
        RETURN
        END


      BLOCK DATA CLDPARAMS

      PARAMETER (NBANDS = 29)

      COMMON /CLDOPTPROPS/ EXTLIQ1(58,NBANDS),
     &     SSALIQ1(58,NBANDS), ASYLIQ1(58,NBANDS),
     &     EXTICE2(43,NBANDS), SSAICE2(43,NBANDS),
     &     ASYICE2(43,NBANDS), 
     &     EXTICE3(46,NBANDS), SSAICE3(46,NBANDS),
     &     ASYICE3(46,NBANDS), FDLICE3(46,NBANDS)
         

C     Everything below is for INFLAG = 2.
      DATA (EXTLIQ1(I,16),I=1,58) /
C     BAND 16
     &8.981463E-01,6.317895E-01,4.557508E-01,3.481624E-01,2.797950E-01,
     &2.342753E-01,2.026934E-01,1.800102E-01,1.632408E-01,1.505384E-01,
     &1.354524E-01,1.246520E-01,1.154342E-01,1.074756E-01,1.005353E-01,
     &9.442987E-02,8.901760E-02,8.418693E-02,7.984904E-02,7.593229E-02,
     &7.237827E-02,6.913887E-02,6.617415E-02,6.345061E-02,6.094001E-02,
     &5.861834E-02,5.646506E-02,5.446250E-02,5.249596E-02,5.081114E-02,
     &4.922243E-02,4.772189E-02,4.630243E-02,4.495766E-02,4.368189E-02,
     &4.246995E-02,4.131720E-02,4.021941E-02,3.917276E-02,3.817376E-02,
     &3.721926E-02,3.630635E-02,3.543237E-02,3.459491E-02,3.379171E-02,
     &3.302073E-02,3.228007E-02,3.156798E-02,3.088284E-02,3.022315E-02,
     &2.958753E-02,2.897468E-02,2.838340E-02,2.781258E-02,2.726117E-02,
     &2.672821E-02,2.621278E-02,2.5714E-02/
      DATA (EXTLIQ1(I,17),I=1,58) /
C     BAND 17
     &8.293797E-01,6.048371E-01,4.465706E-01,3.460387E-01,2.800064E-01,
     &2.346584E-01,2.022399E-01,1.782626E-01,1.600153E-01,1.457903E-01,
     &1.334061E-01,1.228548E-01,1.138396E-01,1.060486E-01,9.924856E-02,
     &9.326208E-02,8.795158E-02,8.320883E-02,7.894750E-02,7.509792E-02,
     &7.160323E-02,6.841653E-02,6.549889E-02,6.281763E-02,6.034516E-02,
     &5.805802E-02,5.593615E-02,5.396226E-02,5.202302E-02,5.036246E-02,
     &4.879606E-02,4.731610E-02,4.591565E-02,4.458852E-02,4.332912E-02,
     &4.213243E-02,4.099390E-02,3.990941E-02,3.887522E-02,3.788792E-02,
     &3.694440E-02,3.604183E-02,3.517760E-02,3.434934E-02,3.355485E-02,
     &3.279211E-02,3.205925E-02,3.135458E-02,3.067648E-02,3.002349E-02,
     &2.939425E-02,2.878748E-02,2.820200E-02,2.763673E-02,2.709062E-02,
     &2.656272E-02,2.605214E-02,2.5558E-02/
      DATA (EXTLIQ1(I,18),I=1,58) /
C     BAND 18
     &9.193685E-01,6.128292E-01,4.344150E-01,3.303048E-01,2.659500E-01,
     &2.239727E-01,1.953457E-01,1.751012E-01,1.603515E-01,1.493360E-01,
     &1.323791E-01,1.219335E-01,1.130076E-01,1.052926E-01,9.855839E-02,
     &9.262925E-02,8.736918E-02,8.267112E-02,7.844965E-02,7.463585E-02,
     &7.117343E-02,6.801601E-02,6.512503E-02,6.246815E-02,6.001806E-02,
     &5.775154E-02,5.564872E-02,5.369250E-02,5.176284E-02,5.011536E-02,
     &4.856099E-02,4.709211E-02,4.570193E-02,4.438430E-02,4.313375E-02,
     &4.194529E-02,4.081443E-02,3.973712E-02,3.870966E-02,3.772866E-02,
     &3.679108E-02,3.589409E-02,3.503514E-02,3.421185E-02,3.342206E-02,
     &3.266377E-02,3.193513E-02,3.123447E-02,3.056018E-02,2.991081E-02,
     &2.928502E-02,2.868154E-02,2.809920E-02,2.753692E-02,2.699367E-02,
     &2.646852E-02,2.596057E-02,2.5469E-02/
      DATA (EXTLIQ1(I,19),I=1,58) /
C     BAND 19
     &9.136931E-01,5.743244E-01,4.080708E-01,3.150572E-01,2.577261E-01,
     &2.197900E-01,1.933037E-01,1.740212E-01,1.595056E-01,1.482756E-01,
     &1.312164E-01,1.209246E-01,1.121227E-01,1.045095E-01,9.785967E-02,
     &9.200149E-02,8.680170E-02,8.215531E-02,7.797850E-02,7.420361E-02,
     &7.077530E-02,6.764798E-02,6.478369E-02,6.215063E-02,5.972189E-02,
     &5.747458E-02,5.538913E-02,5.344866E-02,5.153216E-02,4.989745E-02,
     &4.835476E-02,4.689661E-02,4.551629E-02,4.420777E-02,4.296563E-02,
     &4.178497E-02,4.066137E-02,3.959081E-02,3.856963E-02,3.759452E-02,
     &3.666244E-02,3.577061E-02,3.491650E-02,3.409777E-02,3.331227E-02,
     &3.255803E-02,3.183322E-02,3.113617E-02,3.046530E-02,2.981918E-02,
     &2.919646E-02,2.859591E-02,2.801635E-02,2.745671E-02,2.691599E-02,
     &2.639324E-02,2.588759E-02,2.5398E-02/
      DATA (EXTLIQ1(I,20),I=1,58) /
C     BAND 20
     &8.447548E-01,5.326840E-01,3.921523E-01,3.119082E-01,2.597055E-01,
     &2.228737E-01,1.954157E-01,1.741155E-01,1.570881E-01,1.431520E-01,
     &1.302034E-01,1.200491E-01,1.113571E-01,1.038330E-01,9.725657E-02,
     &9.145949E-02,8.631112E-02,8.170840E-02,7.756901E-02,7.382641E-02,
     &7.042616E-02,6.732338E-02,6.448069E-02,6.186672E-02,5.945494E-02,
     &5.722277E-02,5.515089E-02,5.322262E-02,5.132153E-02,4.969799E-02,
     &4.816556E-02,4.671686E-02,4.534525E-02,4.404480E-02,4.281014E-02,
     &4.163643E-02,4.051930E-02,3.945479E-02,3.843927E-02,3.746945E-02,
     &3.654234E-02,3.565518E-02,3.480547E-02,3.399088E-02,3.320930E-02,
     &3.245876E-02,3.173745E-02,3.104371E-02,3.037600E-02,2.973287E-02,
     &2.911300E-02,2.851516E-02,2.793818E-02,2.738101E-02,2.684264E-02,
     &2.632214E-02,2.581863E-02,2.5331E-02/
      DATA (EXTLIQ1(I,21),I=1,58) /
C     BAND 21
     &7.727642E-01,5.034865E-01,3.808673E-01,3.080333E-01,2.586453E-01,
     &2.224989E-01,1.947060E-01,1.725821E-01,1.545096E-01,1.394456E-01,
     &1.288683E-01,1.188852E-01,1.103317E-01,1.029214E-01,9.643967E-02,
     &9.072239E-02,8.564194E-02,8.109758E-02,7.700875E-02,7.331026E-02,
     &6.994879E-02,6.688028E-02,6.406807E-02,6.148133E-02,5.909400E-02,
     &5.688388E-02,5.483197E-02,5.292185E-02,5.103763E-02,4.942905E-02,
     &4.791039E-02,4.647438E-02,4.511453E-02,4.382497E-02,4.260043E-02,
     &4.143616E-02,4.032784E-02,3.927155E-02,3.826375E-02,3.730117E-02,
     &3.638087E-02,3.550013E-02,3.465646E-02,3.384759E-02,3.307141E-02,
     &3.232598E-02,3.160953E-02,3.092040E-02,3.025706E-02,2.961810E-02,
     &2.900220E-02,2.840814E-02,2.783478E-02,2.728106E-02,2.674599E-02,
     &2.622864E-02,2.572816E-02,2.5244E-02/
      DATA (EXTLIQ1(I,22),I=1,58) /
C     BAND 22
     &7.416833E-01,4.959591E-01,3.775057E-01,3.056353E-01,2.565943E-01,
     &2.206935E-01,1.931479E-01,1.712860E-01,1.534837E-01,1.386906E-01,
     &1.281198E-01,1.182344E-01,1.097595E-01,1.024137E-01,9.598552E-02,
     &9.031320E-02,8.527093E-02,8.075927E-02,7.669869E-02,7.302481E-02,
     &6.968491E-02,6.663542E-02,6.384008E-02,6.126838E-02,5.889452E-02,
     &5.669654E-02,5.465558E-02,5.275540E-02,5.087937E-02,4.927904E-02,
     &4.776796E-02,4.633895E-02,4.498557E-02,4.370202E-02,4.248306E-02,
     &4.132399E-02,4.022052E-02,3.916878E-02,3.816523E-02,3.720665E-02,
     &3.629011E-02,3.541290E-02,3.457257E-02,3.376685E-02,3.299365E-02,
     &3.225105E-02,3.153728E-02,3.085069E-02,3.018977E-02,2.955310E-02,
     &2.893940E-02,2.834742E-02,2.777606E-02,2.722424E-02,2.669099E-02,
     &2.617539E-02,2.567658E-02,2.5194E-02/
      DATA (EXTLIQ1(I,23),I=1,58) /
C     BAND 23
     &7.058580E-01,4.866573E-01,3.712238E-01,2.998638E-01,2.513441E-01,
     &2.161972E-01,1.895576E-01,1.686669E-01,1.518437E-01,1.380046E-01,
     &1.267564E-01,1.170399E-01,1.087026E-01,1.014704E-01,9.513729E-02,
     &8.954555E-02,8.457221E-02,8.012009E-02,7.611136E-02,7.248294E-02,
     &6.918317E-02,6.616934E-02,6.340584E-02,6.086273E-02,5.851465E-02,
     &5.634001E-02,5.432027E-02,5.243946E-02,5.058070E-02,4.899628E-02,
     &4.749975E-02,4.608411E-02,4.474303E-02,4.347082E-02,4.226237E-02,
     &4.111303E-02,4.001861E-02,3.897528E-02,3.797959E-02,3.702835E-02,
     &3.611867E-02,3.524791E-02,3.441364E-02,3.361360E-02,3.284577E-02,
     &3.210823E-02,3.139923E-02,3.071716E-02,3.006052E-02,2.942791E-02,
     &2.881806E-02,2.822974E-02,2.766185E-02,2.711335E-02,2.658326E-02,
     &2.607066E-02,2.557473E-02,2.5095E-02/
      DATA (EXTLIQ1(I,24),I=1,58) /
C     BAND 24
     &6.822779E-01,4.750373E-01,3.634834E-01,2.940726E-01,2.468060E-01,
     &2.125768E-01,1.866586E-01,1.663588E-01,1.500326E-01,1.366192E-01,
     &1.253472E-01,1.158052E-01,1.076101E-01,1.004954E-01,9.426089E-02,
     &8.875268E-02,8.385090E-02,7.946063E-02,7.550578E-02,7.192466E-02,
     &6.866669E-02,6.569001E-02,6.295971E-02,6.044642E-02,5.812526E-02,
     &5.597500E-02,5.397746E-02,5.211690E-02,5.027505E-02,4.870703E-02,
     &4.722555E-02,4.582373E-02,4.449540E-02,4.323497E-02,4.203742E-02,
     &4.089821E-02,3.981321E-02,3.877867E-02,3.779118E-02,3.684762E-02,
     &3.594514E-02,3.508114E-02,3.425322E-02,3.345917E-02,3.269698E-02,
     &3.196477E-02,3.126082E-02,3.058352E-02,2.993141E-02,2.930310E-02,
     &2.869732E-02,2.811289E-02,2.754869E-02,2.700371E-02,2.647698E-02,
     &2.596760E-02,2.547473E-02,2.4998E-02/
      DATA (EXTLIQ1(I,25),I=1,58) /
C     BAND 25
     &6.666233E-01,4.662044E-01,3.579517E-01,2.902984E-01,2.440475E-01,
     &2.104431E-01,1.849277E-01,1.648970E-01,1.487555E-01,1.354714E-01,
     &1.244173E-01,1.149913E-01,1.068903E-01,9.985323E-02,9.368351E-02,
     &8.823009E-02,8.337507E-02,7.902511E-02,7.510529E-02,7.155482E-02,
     &6.832386E-02,6.537113E-02,6.266218E-02,6.016802E-02,5.786408E-02,
     &5.572939E-02,5.374598E-02,5.189830E-02,5.006825E-02,4.851081E-02,
     &4.703906E-02,4.564623E-02,4.432621E-02,4.307349E-02,4.188312E-02,
     &4.075060E-02,3.967183E-02,3.864313E-02,3.766111E-02,3.672269E-02,
     &3.582505E-02,3.496559E-02,3.414196E-02,3.335198E-02,3.259362E-02,
     &3.186505E-02,3.116454E-02,3.049052E-02,2.984152E-02,2.921617E-02,
     &2.861322E-02,2.803148E-02,2.746986E-02,2.692733E-02,2.640295E-02,
     &2.589582E-02,2.540510E-02,2.4930E-02/
      DATA (EXTLIQ1(I,26),I=1,58) /
C     BAND 26
     &6.535669E-01,4.585865E-01,3.529226E-01,2.867245E-01,2.413848E-01,
     &2.083956E-01,1.833191E-01,1.636150E-01,1.477247E-01,1.346392E-01,
     &1.236449E-01,1.143095E-01,1.062828E-01,9.930773E-02,9.319029E-02,
     &8.778150E-02,8.296497E-02,7.864847E-02,7.475799E-02,7.123343E-02,
     &6.802549E-02,6.509332E-02,6.240285E-02,5.992538E-02,5.763657E-02,
     &5.551566E-02,5.354483E-02,5.170870E-02,4.988866E-02,4.834061E-02,
     &4.687751E-02,4.549264E-02,4.417999E-02,4.293410E-02,4.175006E-02,
     &4.062344E-02,3.955019E-02,3.852663E-02,3.754943E-02,3.661553E-02,
     &3.572214E-02,3.486669E-02,3.404683E-02,3.326040E-02,3.250542E-02,
     &3.178003E-02,3.108254E-02,3.041139E-02,2.976511E-02,2.914235E-02,
     &2.854187E-02,2.796247E-02,2.740309E-02,2.686271E-02,2.634038E-02,
     &2.583520E-02,2.534636E-02,2.4873E-02/
      DATA (EXTLIQ1(I,27),I=1,58) /
C     BAND 27
     &6.448790E-01,4.541425E-01,3.503348E-01,2.850494E-01,2.401966E-01,
     &2.074811E-01,1.825631E-01,1.629515E-01,1.471142E-01,1.340574E-01,
     &1.231462E-01,1.138628E-01,1.058802E-01,9.894286E-02,9.285818E-02,
     &8.747802E-02,8.268676E-02,7.839271E-02,7.452230E-02,7.101580E-02,
     &6.782418E-02,6.490685E-02,6.222991E-02,5.976484E-02,5.748742E-02,
     &5.537703E-02,5.341593E-02,5.158883E-02,4.977355E-02,4.823172E-02,
     &4.677430E-02,4.539465E-02,4.408680E-02,4.284533E-02,4.166539E-02,
     &4.054257E-02,3.947283E-02,3.845256E-02,3.747842E-02,3.654737E-02,
     &3.565665E-02,3.480370E-02,3.398620E-02,3.320198E-02,3.244908E-02,
     &3.172566E-02,3.103002E-02,3.036062E-02,2.971600E-02,2.909482E-02,
     &2.849582E-02,2.791785E-02,2.735982E-02,2.682072E-02,2.629960E-02,
     &2.579559E-02,2.530786E-02,2.4836E-02/
      DATA (EXTLIQ1(I,28),I=1,58) /
C     BAND 28
     &6.422688E-01,4.528453E-01,3.497232E-01,2.847724E-01,2.400815E-01,
     &2.074403E-01,1.825502E-01,1.629415E-01,1.470934E-01,1.340183E-01,
     &1.230935E-01,1.138049E-01,1.058201E-01,9.888245E-02,9.279878E-02,
     &8.742053E-02,8.263175E-02,7.834058E-02,7.447327E-02,7.097000E-02,
     &6.778167E-02,6.486765E-02,6.219400E-02,5.973215E-02,5.745790E-02,
     &5.535059E-02,5.339250E-02,5.156831E-02,4.975308E-02,4.821235E-02,
     &4.675596E-02,4.537727E-02,4.407030E-02,4.282968E-02,4.165053E-02,
     &4.052845E-02,3.945941E-02,3.843980E-02,3.746628E-02,3.653583E-02,
     &3.564567E-02,3.479326E-02,3.397626E-02,3.319253E-02,3.244008E-02,
     &3.171711E-02,3.102189E-02,3.035289E-02,2.970866E-02,2.908784E-02,
     &2.848920E-02,2.791156E-02,2.735385E-02,2.681507E-02,2.629425E-02,
     &2.579053E-02,2.530308E-02,2.4831E-02/
      DATA (EXTLIQ1(I,29),I=1,58) /
C     BAND 29
     &4.614710E-01,4.556116E-01,4.056568E-01,3.529833E-01,3.060334E-01,
     &2.658127E-01,2.316095E-01,2.024325E-01,1.773749E-01,1.556867E-01,
     &1.455558E-01,1.332882E-01,1.229052E-01,1.140067E-01,1.062981E-01,
     &9.955703E-02,9.361333E-02,8.833420E-02,8.361467E-02,7.937071E-02,
     &7.553420E-02,7.204942E-02,6.887031E-02,6.595851E-02,6.328178E-02,
     &6.081286E-02,5.852854E-02,5.640892E-02,5.431269E-02,5.252561E-02,
     &5.084345E-02,4.925727E-02,4.775910E-02,4.634182E-02,4.499907E-02,
     &4.372512E-02,4.251484E-02,4.136357E-02,4.026710E-02,3.922162E-02,
     &3.822365E-02,3.727004E-02,3.635790E-02,3.548457E-02,3.464764E-02,
     &3.384488E-02,3.307424E-02,3.233384E-02,3.162192E-02,3.093688E-02,
     &3.027723E-02,2.964158E-02,2.902864E-02,2.843722E-02,2.786621E-02,
     &2.731457E-02,2.678133E-02,2.6266E-02/
      DATA (SSALIQ1(I,16),I=1,58) /
C     BAND 16
     &8.143821E-01,7.836739E-01,7.550722E-01,7.306269E-01,7.105612E-01,
     &6.946649E-01,6.825556E-01,6.737762E-01,6.678448E-01,6.642830E-01,
     &6.679741E-01,6.584607E-01,6.505598E-01,6.440951E-01,6.388901E-01,
     &6.347689E-01,6.315549E-01,6.290718E-01,6.271432E-01,6.255928E-01,
     &6.242441E-01,6.229207E-01,6.214464E-01,6.196445E-01,6.173388E-01,
     &6.143527E-01,6.105099E-01,6.056339E-01,6.108290E-01,6.073939E-01,
     &6.043073E-01,6.015473E-01,5.990913E-01,5.969173E-01,5.950028E-01,
     &5.933257E-01,5.918636E-01,5.905944E-01,5.894957E-01,5.885453E-01,
     &5.877209E-01,5.870003E-01,5.863611E-01,5.857811E-01,5.852381E-01,
     &5.847098E-01,5.841738E-01,5.836081E-01,5.829901E-01,5.822979E-01,
     &5.815089E-01,5.806011E-01,5.795521E-01,5.783396E-01,5.769413E-01,
     &5.753351E-01,5.734986E-01,5.7141E-01/
      DATA (SSALIQ1(I,17),I=1,58) /
C     BAND 17
     &8.165821E-01,8.002015E-01,7.816921E-01,7.634131E-01,7.463721E-01,
     &7.312469E-01,7.185883E-01,7.088975E-01,7.026671E-01,7.004020E-01,
     &7.042138E-01,6.960930E-01,6.894243E-01,6.840459E-01,6.797957E-01,
     &6.765119E-01,6.740325E-01,6.721955E-01,6.708391E-01,6.698013E-01,
     &6.689201E-01,6.680339E-01,6.669805E-01,6.655982E-01,6.637250E-01,
     &6.611992E-01,6.578588E-01,6.535420E-01,6.584449E-01,6.553992E-01,
     &6.526547E-01,6.501917E-01,6.479905E-01,6.460313E-01,6.442945E-01,
     &6.427605E-01,6.414094E-01,6.402217E-01,6.391775E-01,6.382573E-01,
     &6.374413E-01,6.367099E-01,6.360433E-01,6.354218E-01,6.348257E-01,
     &6.342355E-01,6.336313E-01,6.329935E-01,6.323023E-01,6.315383E-01,
     &6.306814E-01,6.297122E-01,6.286110E-01,6.273579E-01,6.259333E-01,
     &6.243176E-01,6.224910E-01,6.2043E-01/
      DATA (SSALIQ1(I,18),I=1,58) /
C     BAND 18
     &9.900163E-01,9.854307E-01,9.797730E-01,9.733113E-01,9.664245E-01,
     &9.594976E-01,9.529055E-01,9.470112E-01,9.421695E-01,9.387304E-01,
     &9.344918E-01,9.305302E-01,9.267048E-01,9.230072E-01,9.194289E-01,
     &9.159616E-01,9.125968E-01,9.093260E-01,9.061409E-01,9.030330E-01,
     &8.999940E-01,8.970154E-01,8.940888E-01,8.912058E-01,8.883579E-01,
     &8.855368E-01,8.827341E-01,8.799413E-01,8.777423E-01,8.749566E-01,
     &8.722298E-01,8.695605E-01,8.669469E-01,8.643875E-01,8.618806E-01,
     &8.594246E-01,8.570179E-01,8.546589E-01,8.523459E-01,8.500773E-01,
     &8.478516E-01,8.456670E-01,8.435219E-01,8.414148E-01,8.393439E-01,
     &8.373078E-01,8.353047E-01,8.333330E-01,8.313911E-01,8.294774E-01,
     &8.275904E-01,8.257282E-01,8.238893E-01,8.220721E-01,8.202751E-01,
     &8.184965E-01,8.167346E-01,8.1499E-01/
      DATA (SSALIQ1(I,19),I=1,58) /
C     BAND 19
     &9.999916E-01,9.987396E-01,9.966900E-01,9.950738E-01,9.937531E-01,
     &9.925912E-01,9.914525E-01,9.902018E-01,9.887046E-01,9.868263E-01,
     &9.849039E-01,9.832372E-01,9.815265E-01,9.797770E-01,9.779940E-01,
     &9.761827E-01,9.743481E-01,9.724955E-01,9.706303E-01,9.687575E-01,
     &9.668823E-01,9.650100E-01,9.631457E-01,9.612947E-01,9.594622E-01,
     &9.576534E-01,9.558734E-01,9.541275E-01,9.522059E-01,9.504258E-01,
     &9.486459E-01,9.468676E-01,9.450921E-01,9.433208E-01,9.415548E-01,
     &9.397955E-01,9.380441E-01,9.363022E-01,9.345706E-01,9.328510E-01,
     &9.311445E-01,9.294524E-01,9.277761E-01,9.261167E-01,9.244755E-01,
     &9.228540E-01,9.212534E-01,9.196748E-01,9.181197E-01,9.165894E-01,
     &9.150851E-01,9.136080E-01,9.121596E-01,9.107410E-01,9.093536E-01,
     &9.079987E-01,9.066775E-01,9.0539E-01/
      DATA (SSALIQ1(I,20),I=1,58) /
C     BAND 20
     &9.979493E-01,9.964113E-01,9.950014E-01,9.937045E-01,9.924964E-01,
     &9.913546E-01,9.902575E-01,9.891843E-01,9.881136E-01,9.870238E-01,
     &9.859934E-01,9.849372E-01,9.838873E-01,9.828434E-01,9.818052E-01,
     &9.807725E-01,9.797450E-01,9.787225E-01,9.777047E-01,9.766914E-01,
     &9.756823E-01,9.746771E-01,9.736756E-01,9.726775E-01,9.716827E-01,
     &9.706907E-01,9.697014E-01,9.687145E-01,9.678060E-01,9.668108E-01,
     &9.658218E-01,9.648391E-01,9.638629E-01,9.628936E-01,9.619313E-01,
     &9.609763E-01,9.600287E-01,9.590888E-01,9.581569E-01,9.572330E-01,
     &9.563176E-01,9.554108E-01,9.545128E-01,9.536239E-01,9.527443E-01,
     &9.518741E-01,9.510137E-01,9.501633E-01,9.493230E-01,9.484931E-01,
     &9.476740E-01,9.468656E-01,9.460683E-01,9.452824E-01,9.445080E-01,
     &9.437454E-01,9.429948E-01,9.4226E-01/
      DATA (SSALIQ1(I,21),I=1,58) /
C     BAND 21
     &9.988742E-01,9.982668E-01,9.976935E-01,9.971497E-01,9.966314E-01,
     &9.961344E-01,9.956545E-01,9.951873E-01,9.947286E-01,9.942741E-01,
     &9.938457E-01,9.933947E-01,9.929473E-01,9.925032E-01,9.920621E-01,
     &9.916237E-01,9.911875E-01,9.907534E-01,9.903209E-01,9.898898E-01,
     &9.894597E-01,9.890304E-01,9.886015E-01,9.881726E-01,9.877435E-01,
     &9.873138E-01,9.868833E-01,9.864516E-01,9.860698E-01,9.856317E-01,
     &9.851957E-01,9.847618E-01,9.843302E-01,9.839008E-01,9.834739E-01,
     &9.830494E-01,9.826275E-01,9.822083E-01,9.817918E-01,9.813782E-01,
     &9.809675E-01,9.805598E-01,9.801552E-01,9.797538E-01,9.793556E-01,
     &9.789608E-01,9.785695E-01,9.781817E-01,9.777975E-01,9.774171E-01,
     &9.770404E-01,9.766676E-01,9.762988E-01,9.759340E-01,9.755733E-01,
     &9.752169E-01,9.748649E-01,9.7452E-01/
      DATA (SSALIQ1(I,22),I=1,58) /
C     BAND 22
     &9.994441E-01,9.991608E-01,9.988949E-01,9.986439E-01,9.984054E-01,
     &9.981768E-01,9.979557E-01,9.977396E-01,9.975258E-01,9.973120E-01,
     &9.971011E-01,9.968852E-01,9.966708E-01,9.964578E-01,9.962462E-01,
     &9.960357E-01,9.958264E-01,9.956181E-01,9.954108E-01,9.952043E-01,
     &9.949987E-01,9.947937E-01,9.945892E-01,9.943853E-01,9.941818E-01,
     &9.939786E-01,9.937757E-01,9.935728E-01,9.933922E-01,9.931825E-01,
     &9.929739E-01,9.927661E-01,9.925592E-01,9.923534E-01,9.921485E-01,
     &9.919447E-01,9.917421E-01,9.915406E-01,9.913403E-01,9.911412E-01,
     &9.909435E-01,9.907470E-01,9.905519E-01,9.903581E-01,9.901659E-01,
     &9.899751E-01,9.897858E-01,9.895981E-01,9.894120E-01,9.892276E-01,
     &9.890447E-01,9.888637E-01,9.886845E-01,9.885070E-01,9.883314E-01,
     &9.881576E-01,9.879859E-01,9.8782E-01/
      DATA (SSALIQ1(I,23),I=1,58) /
C     BAND 23
     &9.999138E-01,9.998730E-01,9.998338E-01,9.997965E-01,9.997609E-01,
     &9.997270E-01,9.996944E-01,9.996629E-01,9.996321E-01,9.996016E-01,
     &9.995690E-01,9.995372E-01,9.995057E-01,9.994744E-01,9.994433E-01,
     &9.994124E-01,9.993817E-01,9.993510E-01,9.993206E-01,9.992903E-01,
     &9.992600E-01,9.992299E-01,9.991998E-01,9.991698E-01,9.991398E-01,
     &9.991098E-01,9.990799E-01,9.990499E-01,9.990231E-01,9.989920E-01,
     &9.989611E-01,9.989302E-01,9.988996E-01,9.988690E-01,9.988386E-01,
     &9.988084E-01,9.987783E-01,9.987485E-01,9.987187E-01,9.986891E-01,
     &9.986598E-01,9.986306E-01,9.986017E-01,9.985729E-01,9.985443E-01,
     &9.985160E-01,9.984879E-01,9.984600E-01,9.984324E-01,9.984050E-01,
     &9.983778E-01,9.983509E-01,9.983243E-01,9.982980E-01,9.982719E-01,
     &9.982461E-01,9.982206E-01,9.9820E-01/
      DATA (SSALIQ1(I,24),I=1,58) /
C     BAND 24
     &9.999985E-01,9.999979E-01,9.999972E-01,9.999966E-01,9.999961E-01,
     &9.999955E-01,9.999950E-01,9.999944E-01,9.999938E-01,9.999933E-01,
     &9.999927E-01,9.999921E-01,9.999915E-01,9.999910E-01,9.999904E-01,
     &9.999899E-01,9.999893E-01,9.999888E-01,9.999882E-01,9.999877E-01,
     &9.999871E-01,9.999866E-01,9.999861E-01,9.999855E-01,9.999850E-01,
     &9.999844E-01,9.999839E-01,9.999833E-01,9.999828E-01,9.999823E-01,
     &9.999817E-01,9.999812E-01,9.999807E-01,9.999801E-01,9.999796E-01,
     &9.999791E-01,9.999786E-01,9.999781E-01,9.999776E-01,9.999770E-01,
     &9.999765E-01,9.999761E-01,9.999756E-01,9.999751E-01,9.999746E-01,
     &9.999741E-01,9.999736E-01,9.999732E-01,9.999727E-01,9.999722E-01,
     &9.999718E-01,9.999713E-01,9.999709E-01,9.999705E-01,9.999701E-01,
     &9.999697E-01,9.999692E-01,9.9997E-01/
      DATA (SSALIQ1(I,25),I=1,58) /
C     BAND 25
     &9.999999E-01,9.999998E-01,9.999997E-01,9.999997E-01,9.999997E-01,
     &9.999996E-01,9.999996E-01,9.999995E-01,9.999995E-01,9.999994E-01,
     &9.999994E-01,9.999993E-01,9.999993E-01,9.999992E-01,9.999992E-01,
     &9.999991E-01,9.999991E-01,9.999991E-01,9.999990E-01,9.999989E-01,
     &9.999989E-01,9.999989E-01,9.999988E-01,9.999988E-01,9.999987E-01,
     &9.999987E-01,9.999986E-01,9.999986E-01,9.999985E-01,9.999985E-01,
     &9.999984E-01,9.999984E-01,9.999984E-01,9.999983E-01,9.999983E-01,
     &9.999982E-01,9.999982E-01,9.999982E-01,9.999981E-01,9.999980E-01,
     &9.999980E-01,9.999980E-01,9.999979E-01,9.999979E-01,9.999978E-01,
     &9.999978E-01,9.999977E-01,9.999977E-01,9.999977E-01,9.999976E-01,
     &9.999976E-01,9.999975E-01,9.999975E-01,9.999974E-01,9.999974E-01,
     &9.999974E-01,9.999973E-01,1.0000E+00/
      DATA (SSALIQ1(I,26),I=1,58) /
C     BAND 26
     &9.999997E-01,9.999995E-01,9.999993E-01,9.999992E-01,9.999990E-01,
     &9.999989E-01,9.999988E-01,9.999987E-01,9.999986E-01,9.999985E-01,
     &9.999984E-01,9.999983E-01,9.999982E-01,9.999981E-01,9.999980E-01,
     &9.999978E-01,9.999977E-01,9.999976E-01,9.999975E-01,9.999974E-01,
     &9.999973E-01,9.999972E-01,9.999970E-01,9.999969E-01,9.999968E-01,
     &9.999967E-01,9.999966E-01,9.999965E-01,9.999964E-01,9.999963E-01,
     &9.999962E-01,9.999961E-01,9.999959E-01,9.999958E-01,9.999957E-01,
     &9.999956E-01,9.999955E-01,9.999954E-01,9.999953E-01,9.999952E-01,
     &9.999951E-01,9.999949E-01,9.999949E-01,9.999947E-01,9.999946E-01,
     &9.999945E-01,9.999944E-01,9.999943E-01,9.999942E-01,9.999941E-01,
     &9.999940E-01,9.999939E-01,9.999938E-01,9.999937E-01,9.999936E-01,
     &9.999935E-01,9.999934E-01,9.9999E-01/
      DATA (SSALIQ1(I,27),I=1,58) /
C     BAND 27
     &9.999984E-01,9.999976E-01,9.999969E-01,9.999962E-01,9.999956E-01,
     &9.999950E-01,9.999945E-01,9.999940E-01,9.999935E-01,9.999931E-01,
     &9.999926E-01,9.999920E-01,9.999914E-01,9.999908E-01,9.999903E-01,
     &9.999897E-01,9.999891E-01,9.999886E-01,9.999880E-01,9.999874E-01,
     &9.999868E-01,9.999863E-01,9.999857E-01,9.999851E-01,9.999846E-01,
     &9.999840E-01,9.999835E-01,9.999829E-01,9.999824E-01,9.999818E-01,
     &9.999812E-01,9.999806E-01,9.999800E-01,9.999795E-01,9.999789E-01,
     &9.999783E-01,9.999778E-01,9.999773E-01,9.999767E-01,9.999761E-01,
     &9.999756E-01,9.999750E-01,9.999745E-01,9.999739E-01,9.999734E-01,
     &9.999729E-01,9.999723E-01,9.999718E-01,9.999713E-01,9.999708E-01,
     &9.999703E-01,9.999697E-01,9.999692E-01,9.999687E-01,9.999683E-01,
     &9.999678E-01,9.999673E-01,9.9997E-01/
      DATA (SSALIQ1(I,28),I=1,58) /
C     BAND 28
     &9.999981E-01,9.999973E-01,9.999965E-01,9.999958E-01,9.999951E-01,
     &9.999943E-01,9.999937E-01,9.999930E-01,9.999924E-01,9.999918E-01,
     &9.999912E-01,9.999905E-01,9.999897E-01,9.999890E-01,9.999883E-01,
     &9.999876E-01,9.999869E-01,9.999862E-01,9.999855E-01,9.999847E-01,
     &9.999840E-01,9.999834E-01,9.999827E-01,9.999819E-01,9.999812E-01,
     &9.999805E-01,9.999799E-01,9.999791E-01,9.999785E-01,9.999778E-01,
     &9.999771E-01,9.999764E-01,9.999757E-01,9.999750E-01,9.999743E-01,
     &9.999736E-01,9.999729E-01,9.999722E-01,9.999715E-01,9.999709E-01,
     &9.999701E-01,9.999695E-01,9.999688E-01,9.999682E-01,9.999675E-01,
     &9.999669E-01,9.999662E-01,9.999655E-01,9.999649E-01,9.999642E-01,
     &9.999636E-01,9.999630E-01,9.999624E-01,9.999618E-01,9.999612E-01,
     &9.999606E-01,9.999600E-01,9.9996E-01/
      DATA (SSALIQ1(I,29),I=1,58) /
C     BAND 29
     &8.505737E-01,8.465102E-01,8.394829E-01,8.279508E-01,8.110806E-01,
     &7.900397E-01,7.669615E-01,7.444422E-01,7.253055E-01,7.124831E-01,
     &7.016434E-01,6.885485E-01,6.767340E-01,6.661029E-01,6.565577E-01,
     &6.480013E-01,6.403373E-01,6.334697E-01,6.273034E-01,6.217440E-01,
     &6.166983E-01,6.120740E-01,6.077796E-01,6.037249E-01,5.998207E-01,
     &5.959788E-01,5.921123E-01,5.881354E-01,5.891285E-01,5.851143E-01,
     &5.814653E-01,5.781606E-01,5.751792E-01,5.724998E-01,5.701016E-01,
     &5.679634E-01,5.660642E-01,5.643829E-01,5.628984E-01,5.615898E-01,
     &5.604359E-01,5.594158E-01,5.585083E-01,5.576924E-01,5.569470E-01,
     &5.562512E-01,5.555838E-01,5.549239E-01,5.542503E-01,5.535420E-01,
     &5.527781E-01,5.519374E-01,5.509989E-01,5.499417E-01,5.487445E-01,
     &5.473865E-01,5.458466E-01,5.4410E-01/
      DATA (ASYLIQ1(I,16),I=1,58) /
C     BAND 16
     &8.133297E-01,8.133528E-01,8.173865E-01,8.243205E-01,8.333063E-01,
     &8.436317E-01,8.546611E-01,8.657934E-01,8.764345E-01,8.859837E-01,
     &8.627394E-01,8.824569E-01,8.976887E-01,9.089541E-01,9.167699E-01,
     &9.216517E-01,9.241147E-01,9.246743E-01,9.238469E-01,9.221504E-01,
     &9.201045E-01,9.182299E-01,9.170491E-01,9.170862E-01,9.188653E-01,
     &9.229111E-01,9.297468E-01,9.398950E-01,9.203269E-01,9.260693E-01,
     &9.309373E-01,9.349918E-01,9.382935E-01,9.409030E-01,9.428809E-01,
     &9.442881E-01,9.451851E-01,9.456331E-01,9.456926E-01,9.454247E-01,
     &9.448902E-01,9.441503E-01,9.432661E-01,9.422987E-01,9.413094E-01,
     &9.403594E-01,9.395102E-01,9.388230E-01,9.383594E-01,9.381810E-01,
     &9.383489E-01,9.389251E-01,9.399707E-01,9.415475E-01,9.437167E-01,
     &9.465399E-01,9.500786E-01,9.5439E-01/
      DATA (ASYLIQ1(I,17),I=1,58) /
C     BAND 17
     &8.794448E-01,8.819306E-01,8.837667E-01,8.853832E-01,8.871010E-01,
     &8.892675E-01,8.922584E-01,8.964666E-01,9.022940E-01,9.101456E-01,
     &8.839999E-01,9.035610E-01,9.184568E-01,9.292315E-01,9.364282E-01,
     &9.405887E-01,9.422554E-01,9.419703E-01,9.402759E-01,9.377159E-01,
     &9.348345E-01,9.321769E-01,9.302888E-01,9.297166E-01,9.310075E-01,
     &9.347080E-01,9.413643E-01,9.515216E-01,9.306286E-01,9.361781E-01,
     &9.408374E-01,9.446692E-01,9.477363E-01,9.501013E-01,9.518268E-01,
     &9.529756E-01,9.536105E-01,9.537938E-01,9.535886E-01,9.530574E-01,
     &9.522633E-01,9.512688E-01,9.501370E-01,9.489306E-01,9.477126E-01,
     &9.465459E-01,9.454934E-01,9.446183E-01,9.439833E-01,9.436519E-01,
     &9.436866E-01,9.441508E-01,9.451073E-01,9.466195E-01,9.487501E-01,
     &9.515621E-01,9.551185E-01,9.5948E-01/
      DATA (ASYLIQ1(I,18),I=1,58) /
C     BAND 18
     &8.478817E-01,8.269312E-01,8.161352E-01,8.135960E-01,8.173586E-01,
     &8.254167E-01,8.357072E-01,8.461167E-01,8.544952E-01,8.586776E-01,
     &8.335562E-01,8.524273E-01,8.669052E-01,8.775014E-01,8.847277E-01,
     &8.890958E-01,8.911173E-01,8.913038E-01,8.901669E-01,8.882182E-01,
     &8.859692E-01,8.839315E-01,8.826164E-01,8.825356E-01,8.842004E-01,
     &8.881223E-01,8.948131E-01,9.047837E-01,8.855951E-01,8.911796E-01,
     &8.959229E-01,8.998837E-01,9.031209E-01,9.056939E-01,9.076609E-01,
     &9.090812E-01,9.100134E-01,9.105167E-01,9.106496E-01,9.104712E-01,
     &9.100404E-01,9.094159E-01,9.086568E-01,9.078218E-01,9.069697E-01,
     &9.061595E-01,9.054499E-01,9.048999E-01,9.045683E-01,9.045142E-01,
     &9.047962E-01,9.054730E-01,9.066037E-01,9.082472E-01,9.104623E-01,
     &9.133079E-01,9.168427E-01,9.2113E-01/
      DATA (ASYLIQ1(I,19),I=1,58) /
C     BAND 19
     &8.216697E-01,7.982871E-01,7.891147E-01,7.909083E-01,8.003833E-01,
     &8.142516E-01,8.292290E-01,8.420356E-01,8.493945E-01,8.480316E-01,
     &8.212381E-01,8.394984E-01,8.534095E-01,8.634813E-01,8.702242E-01,
     &8.741483E-01,8.757638E-01,8.755808E-01,8.741095E-01,8.718604E-01,
     &8.693433E-01,8.670686E-01,8.655464E-01,8.652872E-01,8.668006E-01,
     &8.705973E-01,8.771874E-01,8.870809E-01,8.678284E-01,8.732315E-01,
     &8.778084E-01,8.816166E-01,8.847146E-01,8.871603E-01,8.890116E-01,
     &8.903266E-01,8.911632E-01,8.915796E-01,8.916337E-01,8.913834E-01,
     &8.908869E-01,8.902022E-01,8.893873E-01,8.885001E-01,8.875986E-01,
     &8.867411E-01,8.859852E-01,8.853891E-01,8.850111E-01,8.849089E-01,
     &8.851405E-01,8.857639E-01,8.868372E-01,8.884185E-01,8.905656E-01,
     &8.933368E-01,8.967899E-01,9.0098E-01/
      DATA (ASYLIQ1(I,20),I=1,58) /
C     BAND 20
     &8.063610E-01,7.938147E-01,7.921304E-01,7.985092E-01,8.101339E-01,
     &8.242175E-01,8.379913E-01,8.486920E-01,8.535547E-01,8.498083E-01,
     &8.224849E-01,8.405509E-01,8.542436E-01,8.640770E-01,8.705653E-01,
     &8.742227E-01,8.755630E-01,8.751004E-01,8.733491E-01,8.708231E-01,
     &8.680365E-01,8.655035E-01,8.637381E-01,8.632544E-01,8.645665E-01,
     &8.681885E-01,8.746346E-01,8.844188E-01,8.648180E-01,8.700563E-01,
     &8.744672E-01,8.781087E-01,8.810393E-01,8.833174E-01,8.850011E-01,
     &8.861485E-01,8.868183E-01,8.870687E-01,8.869579E-01,8.865441E-01,
     &8.858857E-01,8.850412E-01,8.840686E-01,8.830263E-01,8.819726E-01,
     &8.809658E-01,8.800642E-01,8.793260E-01,8.788099E-01,8.785737E-01,
     &8.786758E-01,8.791746E-01,8.801283E-01,8.815955E-01,8.836340E-01,
     &8.863024E-01,8.896592E-01,8.9376E-01/
      DATA (ASYLIQ1(I,21),I=1,58) /
C     BAND 21
     &7.885899E-01,7.937172E-01,8.020658E-01,8.123971E-01,8.235502E-01,
     &8.343776E-01,8.437336E-01,8.504711E-01,8.534421E-01,8.514978E-01,
     &8.238888E-01,8.417463E-01,8.552057E-01,8.647853E-01,8.710038E-01,
     &8.743798E-01,8.754319E-01,8.746786E-01,8.726386E-01,8.698303E-01,
     &8.667724E-01,8.639836E-01,8.619823E-01,8.612870E-01,8.624165E-01,
     &8.658893E-01,8.722241E-01,8.819394E-01,8.620216E-01,8.671239E-01,
     &8.713983E-01,8.749032E-01,8.776970E-01,8.798385E-01,8.813860E-01,
     &8.823980E-01,8.829332E-01,8.830500E-01,8.828068E-01,8.822623E-01,
     &8.814750E-01,8.805031E-01,8.794056E-01,8.782407E-01,8.770672E-01,
     &8.759432E-01,8.749275E-01,8.740784E-01,8.734547E-01,8.731146E-01,
     &8.731170E-01,8.735199E-01,8.743823E-01,8.757625E-01,8.777191E-01,
     &8.803105E-01,8.835953E-01,8.8763E-01/
      DATA (ASYLIQ1(I,22),I=1,58) /
C     BAND 22
     &7.811516E-01,7.962229E-01,8.096199E-01,8.212996E-01,8.312212E-01,
     &8.393430E-01,8.456236E-01,8.500214E-01,8.524950E-01,8.530031E-01,
     &8.251485E-01,8.429043E-01,8.562461E-01,8.656954E-01,8.717737E-01,
     &8.750020E-01,8.759022E-01,8.749953E-01,8.728027E-01,8.698461E-01,
     &8.666466E-01,8.637257E-01,8.616047E-01,8.608051E-01,8.618483E-01,
     &8.652557E-01,8.715487E-01,8.812485E-01,8.611645E-01,8.662052E-01,
     &8.704173E-01,8.738594E-01,8.765901E-01,8.786678E-01,8.801517E-01,
     &8.810999E-01,8.815713E-01,8.816246E-01,8.813185E-01,8.807114E-01,
     &8.798621E-01,8.788290E-01,8.776713E-01,8.764470E-01,8.752152E-01,
     &8.740343E-01,8.729631E-01,8.720602E-01,8.713842E-01,8.709936E-01,
     &8.709475E-01,8.713041E-01,8.721221E-01,8.734602E-01,8.753774E-01,
     &8.779319E-01,8.811825E-01,8.8519E-01/
      DATA (ASYLIQ1(I,23),I=1,58) /
C     BAND 23
     &7.865744E-01,8.093340E-01,8.257596E-01,8.369940E-01,8.441574E-01,
     &8.483602E-01,8.507096E-01,8.523139E-01,8.542834E-01,8.577321E-01,
     &8.288960E-01,8.465308E-01,8.597175E-01,8.689830E-01,8.748542E-01,
     &8.778584E-01,8.785222E-01,8.773728E-01,8.749370E-01,8.717419E-01,
     &8.683145E-01,8.651816E-01,8.628704E-01,8.619077E-01,8.628205E-01,
     &8.661356E-01,8.723803E-01,8.820815E-01,8.616715E-01,8.666389E-01,
     &8.707753E-01,8.741398E-01,8.767912E-01,8.787885E-01,8.801908E-01,
     &8.810570E-01,8.814460E-01,8.814167E-01,8.810283E-01,8.803395E-01,
     &8.794095E-01,8.782971E-01,8.770613E-01,8.757610E-01,8.744553E-01,
     &8.732031E-01,8.720634E-01,8.710951E-01,8.703572E-01,8.699086E-01,
     &8.698084E-01,8.701155E-01,8.708887E-01,8.721872E-01,8.740698E-01,
     &8.765957E-01,8.798235E-01,8.8381E-01/
      DATA (ASYLIQ1(I,24),I=1,58) /
C     BAND 24
     &8.069513E-01,8.262939E-01,8.398241E-01,8.486352E-01,8.538213E-01,
     &8.564743E-01,8.576854E-01,8.585455E-01,8.601452E-01,8.635755E-01,
     &8.337383E-01,8.512655E-01,8.643049E-01,8.733896E-01,8.790535E-01,
     &8.818295E-01,8.822518E-01,8.808533E-01,8.781676E-01,8.747284E-01,
     &8.710690E-01,8.677229E-01,8.652236E-01,8.641047E-01,8.648993E-01,
     &8.681413E-01,8.743640E-01,8.841007E-01,8.633558E-01,8.682719E-01,
     &8.723543E-01,8.756621E-01,8.782547E-01,8.801915E-01,8.815318E-01,
     &8.823347E-01,8.826598E-01,8.825663E-01,8.821135E-01,8.813608E-01,
     &8.803674E-01,8.791928E-01,8.778960E-01,8.765366E-01,8.751738E-01,
     &8.738670E-01,8.726755E-01,8.716585E-01,8.708755E-01,8.703856E-01,
     &8.702483E-01,8.705229E-01,8.712687E-01,8.725448E-01,8.744109E-01,
     &8.769260E-01,8.801496E-01,8.8414E-01/
      DATA (ASYLIQ1(I,25),I=1,58) /
C     BAND 25
     &8.252182E-01,8.379244E-01,8.471709E-01,8.535760E-01,8.577540E-01,
     &8.603183E-01,8.618820E-01,8.630578E-01,8.644587E-01,8.666970E-01,
     &8.362159E-01,8.536817E-01,8.666387E-01,8.756240E-01,8.811746E-01,
     &8.838273E-01,8.841191E-01,8.825871E-01,8.797681E-01,8.761992E-01,
     &8.724174E-01,8.689593E-01,8.663623E-01,8.651632E-01,8.658988E-01,
     &8.691064E-01,8.753226E-01,8.850847E-01,8.641620E-01,8.690500E-01,
     &8.731026E-01,8.763795E-01,8.789400E-01,8.808438E-01,8.821503E-01,
     &8.829191E-01,8.832095E-01,8.830813E-01,8.825938E-01,8.818064E-01,
     &8.807787E-01,8.795704E-01,8.782408E-01,8.768493E-01,8.754557E-01,
     &8.741193E-01,8.728995E-01,8.718561E-01,8.710484E-01,8.705360E-01,
     &8.703782E-01,8.706347E-01,8.713650E-01,8.726285E-01,8.744849E-01,
     &8.769933E-01,8.802136E-01,8.8421E-01/
      DATA (ASYLIQ1(I,26),I=1,58) /
C     BAND 26
     &8.370583E-01,8.467920E-01,8.537769E-01,8.585136E-01,8.615034E-01,
     &8.632474E-01,8.642468E-01,8.650026E-01,8.660161E-01,8.677882E-01,
     &8.369760E-01,8.543821E-01,8.672699E-01,8.761782E-01,8.816454E-01,
     &8.842103E-01,8.844114E-01,8.827872E-01,8.798766E-01,8.762179E-01,
     &8.723500E-01,8.688112E-01,8.661403E-01,8.648758E-01,8.655563E-01,
     &8.687206E-01,8.749072E-01,8.846546E-01,8.636289E-01,8.684849E-01,
     &8.725054E-01,8.757501E-01,8.782785E-01,8.801503E-01,8.814249E-01,
     &8.821620E-01,8.824211E-01,8.822620E-01,8.817440E-01,8.809268E-01,
     &8.798699E-01,8.786330E-01,8.772756E-01,8.758572E-01,8.744374E-01,
     &8.730760E-01,8.718323E-01,8.707660E-01,8.699366E-01,8.694039E-01,
     &8.692271E-01,8.694661E-01,8.701803E-01,8.714293E-01,8.732727E-01,
     &8.757702E-01,8.789811E-01,8.8297E-01/
      DATA (ASYLIQ1(I,27),I=1,58) /
C     BAND 27
     &8.430819E-01,8.510060E-01,8.567270E-01,8.606533E-01,8.631934E-01,
     &8.647554E-01,8.657471E-01,8.665760E-01,8.676496E-01,8.693754E-01,
     &8.384298E-01,8.557913E-01,8.686214E-01,8.774605E-01,8.828495E-01,
     &8.853287E-01,8.854393E-01,8.837215E-01,8.807161E-01,8.769639E-01,
     &8.730053E-01,8.693812E-01,8.666321E-01,8.652988E-01,8.659219E-01,
     &8.690419E-01,8.751999E-01,8.849360E-01,8.638013E-01,8.686371E-01,
     &8.726369E-01,8.758605E-01,8.783674E-01,8.802176E-01,8.814705E-01,
     &8.821859E-01,8.824234E-01,8.822429E-01,8.817038E-01,8.808658E-01,
     &8.797887E-01,8.785323E-01,8.771560E-01,8.757196E-01,8.742828E-01,
     &8.729052E-01,8.716467E-01,8.705666E-01,8.697250E-01,8.691812E-01,
     &8.689950E-01,8.692264E-01,8.699346E-01,8.711795E-01,8.730209E-01,
     &8.755181E-01,8.787312E-01,8.8272E-01/
      DATA (ASYLIQ1(I,28),I=1,58) /
C     BAND 28
     &8.452284E-01,8.522700E-01,8.572973E-01,8.607031E-01,8.628802E-01,
     &8.642215E-01,8.651198E-01,8.659679E-01,8.671588E-01,8.690853E-01,
     &8.383803E-01,8.557485E-01,8.685851E-01,8.774303E-01,8.828245E-01,
     &8.853077E-01,8.854207E-01,8.837034E-01,8.806962E-01,8.769398E-01,
     &8.729740E-01,8.693393E-01,8.665761E-01,8.652247E-01,8.658253E-01,
     &8.689182E-01,8.750438E-01,8.847424E-01,8.636140E-01,8.684449E-01,
     &8.724400E-01,8.756589E-01,8.781613E-01,8.800072E-01,8.812559E-01,
     &8.819671E-01,8.822007E-01,8.820165E-01,8.814737E-01,8.806322E-01,
     &8.795518E-01,8.782923E-01,8.769129E-01,8.754737E-01,8.740342E-01,
     &8.726542E-01,8.713934E-01,8.703111E-01,8.694677E-01,8.689222E-01,
     &8.687344E-01,8.689646E-01,8.696715E-01,8.709156E-01,8.727563E-01,
     &8.752531E-01,8.784659E-01,8.8245E-01/
      DATA (ASYLIQ1(I,29),I=1,58) /
C     BAND 29
     &7.800869E-01,8.091120E-01,8.325369E-01,8.466266E-01,8.515495E-01,
     &8.499371E-01,8.456203E-01,8.430521E-01,8.470286E-01,8.625431E-01,
     &8.402261E-01,8.610822E-01,8.776608E-01,8.904485E-01,8.999294E-01,
     &9.065860E-01,9.108995E-01,9.133503E-01,9.144187E-01,9.145855E-01,
     &9.143320E-01,9.141402E-01,9.144933E-01,9.158754E-01,9.187716E-01,
     &9.236677E-01,9.310503E-01,9.414058E-01,9.239108E-01,9.300719E-01,
     &9.353612E-01,9.398378E-01,9.435609E-01,9.465895E-01,9.489829E-01,
     &9.508000E-01,9.521002E-01,9.529424E-01,9.533860E-01,9.534902E-01,
     &9.533143E-01,9.529177E-01,9.523596E-01,9.516997E-01,9.509973E-01,
     &9.503121E-01,9.497037E-01,9.492317E-01,9.489558E-01,9.489356E-01,
     &9.492311E-01,9.499019E-01,9.510077E-01,9.526084E-01,9.547636E-01,
     &9.575331E-01,9.609766E-01,9.6515E-01/
C     Spherical Ice Particle Parameterization
C     EXTINCTION UNITS (EXT COEF/IWC): [(m^-1)/(g m^-3)]
      DATA (EXTICE2(I,16),I=1,43) /
C    BAND 16
     &4.101824e-01,2.435514e-01,1.713697e-01,1.314865e-01,1.063406e-01,
     &8.910701e-02,7.659480e-02,6.711784e-02,5.970353e-02,5.375249e-02,
     &4.887577e-02,4.481025e-02,4.137171e-02,3.842744e-02,3.587948e-02,
     &3.365396e-02,3.169419e-02,2.995593e-02,2.840419e-02,2.701091e-02,
     &2.575336e-02,2.461293e-02,2.357423e-02,2.262443e-02,2.175276e-02,
     &2.095012e-02,2.020875e-02,1.952199e-02,1.888412e-02,1.829018e-02,
     &1.773586e-02,1.721738e-02,1.673144e-02,1.627510e-02,1.584579e-02,
     &1.544122e-02,1.505934e-02,1.469833e-02,1.435654e-02,1.403251e-02,
     &1.372492e-02,1.343255e-02,1.315433e-02/
      DATA (EXTICE2(I,17),I=1,43) /
C    BAND 17
     &3.836650e-01,2.304055e-01,1.637265e-01,1.266681e-01,1.031602e-01,
     &8.695191e-02,7.511544e-02,6.610009e-02,5.900909e-02,5.328833e-02,
     &4.857728e-02,4.463133e-02,4.127880e-02,3.839567e-02,3.589013e-02,
     &3.369280e-02,3.175027e-02,3.002079e-02,2.847121e-02,2.707493e-02,
     &2.581031e-02,2.465962e-02,2.360815e-02,2.264363e-02,2.175571e-02,
     &2.093563e-02,2.017592e-02,1.947015e-02,1.881278e-02,1.819901e-02,
     &1.762463e-02,1.708598e-02,1.657982e-02,1.610330e-02,1.565390e-02,
     &1.522937e-02,1.482768e-02,1.444706e-02,1.408588e-02,1.374270e-02,
     &1.341619e-02,1.310517e-02,1.280857e-02/
      DATA (EXTICE2(I,18),I=1,43) /
C    BAND 18
     &4.152673e-01,2.436816e-01,1.702243e-01,1.299704e-01,1.047528e-01,
     &8.756039e-02,7.513327e-02,6.575690e-02,5.844616e-02,5.259609e-02,
     &4.781531e-02,4.383980e-02,4.048517e-02,3.761891e-02,3.514342e-02,
     &3.298525e-02,3.108814e-02,2.940825e-02,2.791096e-02,2.656858e-02,
     &2.535869e-02,2.426297e-02,2.326627e-02,2.235602e-02,2.152164e-02,
     &2.075420e-02,2.004613e-02,1.939091e-02,1.878296e-02,1.821744e-02,
     &1.769015e-02,1.719741e-02,1.673600e-02,1.630308e-02,1.589615e-02,
     &1.551298e-02,1.515159e-02,1.481021e-02,1.448726e-02,1.418131e-02,
     &1.389109e-02,1.361544e-02,1.335330e-02/
      DATA (EXTICE2(I,19),I=1,43) /
C    BAND 19
     &3.873250e-01,2.331609e-01,1.655002e-01,1.277753e-01,1.038247e-01,
     &8.731780e-02,7.527638e-02,6.611873e-02,5.892850e-02,5.313885e-02,
     &4.838068e-02,4.440356e-02,4.103167e-02,3.813804e-02,3.562870e-02,
     &3.343269e-02,3.149539e-02,2.977414e-02,2.823510e-02,2.685112e-02,
     &2.560015e-02,2.446411e-02,2.342805e-02,2.247948e-02,2.160789e-02,
     &2.080438e-02,2.006139e-02,1.937238e-02,1.873177e-02,1.813469e-02,
     &1.757689e-02,1.705468e-02,1.656479e-02,1.610435e-02,1.567081e-02,
     &1.526192e-02,1.487565e-02,1.451020e-02,1.416396e-02,1.383546e-02,
     &1.352339e-02,1.322657e-02,1.294392e-02/
      DATA (EXTICE2(I,20),I=1,43) /
C    BAND 20
     &3.784280e-01,2.291396e-01,1.632551e-01,1.263775e-01,1.028944e-01,
     &8.666975e-02,7.480952e-02,6.577335e-02,5.866714e-02,5.293694e-02,
     &4.822153e-02,4.427547e-02,4.092626e-02,3.804918e-02,3.555184e-02,
     &3.336440e-02,3.143307e-02,2.971577e-02,2.817912e-02,2.679632e-02,
     &2.554558e-02,2.440903e-02,2.337187e-02,2.242173e-02,2.154821e-02,
     &2.074249e-02,1.999706e-02,1.930546e-02,1.866212e-02,1.806221e-02,
     &1.750152e-02,1.697637e-02,1.648352e-02,1.602010e-02,1.558358e-02,
     &1.517172e-02,1.478250e-02,1.441413e-02,1.406498e-02,1.373362e-02,
     &1.341872e-02,1.311911e-02,1.283371e-02/
      DATA (EXTICE2(I,21),I=1,43) /
C    BAND 21
     &3.719909e-01,2.259490e-01,1.613144e-01,1.250648e-01,1.019462e-01,
     &8.595358e-02,7.425064e-02,6.532618e-02,5.830218e-02,5.263421e-02,
     &4.796697e-02,4.405891e-02,4.074013e-02,3.788776e-02,3.541071e-02,
     &3.324008e-02,3.132280e-02,2.961733e-02,2.809071e-02,2.671645e-02,
     &2.547302e-02,2.434276e-02,2.331102e-02,2.236558e-02,2.149614e-02,
     &2.069397e-02,1.995163e-02,1.926272e-02,1.862174e-02,1.802389e-02,
     &1.746500e-02,1.694142e-02,1.644994e-02,1.598772e-02,1.555225e-02,
     &1.514129e-02,1.475286e-02,1.438515e-02,1.403659e-02,1.370572e-02,
     &1.339124e-02,1.309197e-02,1.280685e-02/
      DATA (EXTICE2(I,22),I=1,43) /
C    BAND 22
     &3.713158e-01,2.253816e-01,1.608461e-01,1.246718e-01,1.016109e-01,
     &8.566332e-02,7.399666e-02,6.510199e-02,5.810290e-02,5.245608e-02,
     &4.780702e-02,4.391478e-02,4.060989e-02,3.776982e-02,3.530374e-02,
     &3.314296e-02,3.123458e-02,2.953719e-02,2.801794e-02,2.665043e-02,
     &2.541321e-02,2.428868e-02,2.326224e-02,2.232173e-02,2.145688e-02,
     &2.065899e-02,1.992067e-02,1.923552e-02,1.859808e-02,1.800356e-02,
     &1.744782e-02,1.692721e-02,1.643855e-02,1.597900e-02,1.554606e-02,
     &1.513751e-02,1.475137e-02,1.438586e-02,1.403938e-02,1.371050e-02,
     &1.339793e-02,1.310050e-02,1.281713e-02/
      DATA (EXTICE2(I,23),I=1,43) /
C    BAND 23
     &3.605883e-01,2.204388e-01,1.580431e-01,1.229033e-01,1.004203e-01,
     &8.482616e-02,7.338941e-02,6.465105e-02,5.776176e-02,5.219398e-02,
     &4.760288e-02,4.375369e-02,4.048111e-02,3.766539e-02,3.521771e-02,
     &3.307079e-02,3.117277e-02,2.948303e-02,2.796929e-02,2.660560e-02,
     &2.537086e-02,2.424772e-02,2.322182e-02,2.228114e-02,2.141556e-02,
     &2.061649e-02,1.987661e-02,1.918962e-02,1.855009e-02,1.795330e-02,
     &1.739514e-02,1.687199e-02,1.638069e-02,1.591845e-02,1.548276e-02,
     &1.507143e-02,1.468249e-02,1.431416e-02,1.396486e-02,1.363318e-02,
     &1.331781e-02,1.301759e-02,1.273147e-02/
      DATA (EXTICE2(I,24),I=1,43) /
C    BAND 24
     &3.527890e-01,2.168469e-01,1.560090e-01,1.216216e-01,9.955787e-02,
     &8.421942e-02,7.294827e-02,6.432192e-02,5.751081e-02,5.199888e-02,
     &4.744835e-02,4.362899e-02,4.037847e-02,3.757910e-02,3.514351e-02,
     &3.300546e-02,3.111382e-02,2.942853e-02,2.791775e-02,2.655584e-02,
     &2.532195e-02,2.419892e-02,2.317255e-02,2.223092e-02,2.136402e-02,
     &2.056334e-02,1.982160e-02,1.913258e-02,1.849087e-02,1.789178e-02,
     &1.733124e-02,1.680565e-02,1.631187e-02,1.584711e-02,1.540889e-02,
     &1.499502e-02,1.460354e-02,1.423269e-02,1.388088e-02,1.354670e-02,
     &1.322887e-02,1.292620e-02,1.263767e-02/
      DATA (EXTICE2(I,25),I=1,43) /
C    BAND 25
     &3.477874e-01,2.143515e-01,1.544887e-01,1.205942e-01,9.881779e-02,
     &8.366261e-02,7.251586e-02,6.397790e-02,5.723183e-02,5.176908e-02,
     &4.725658e-02,4.346715e-02,4.024055e-02,3.746055e-02,3.504080e-02,
     &3.291583e-02,3.103507e-02,2.935891e-02,2.785582e-02,2.650042e-02,
     &2.527206e-02,2.415376e-02,2.313142e-02,2.219326e-02,2.132934e-02,
     &2.053122e-02,1.979169e-02,1.910456e-02,1.846448e-02,1.786680e-02,
     &1.730745e-02,1.678289e-02,1.628998e-02,1.582595e-02,1.538835e-02,
     &1.497499e-02,1.458393e-02,1.421341e-02,1.386187e-02,1.352788e-02,
     &1.321019e-02,1.290762e-02,1.261913e-02/
      DATA (EXTICE2(I,26),I=1,43) /
C    BAND 26
     &3.453721e-01,2.130744e-01,1.536698e-01,1.200140e-01,9.838078e-02,
     &8.331940e-02,7.223803e-02,6.374775e-02,5.703770e-02,5.160290e-02,
     &4.711259e-02,4.334110e-02,4.012923e-02,3.736150e-02,3.495208e-02,
     &3.283589e-02,3.096267e-02,2.929302e-02,2.779560e-02,2.644517e-02,
     &2.522119e-02,2.410677e-02,2.308788e-02,2.215281e-02,2.129165e-02,
     &2.049602e-02,1.975874e-02,1.907365e-02,1.843542e-02,1.783943e-02,
     &1.728162e-02,1.675847e-02,1.626685e-02,1.580401e-02,1.536750e-02,
     &1.495515e-02,1.456502e-02,1.419537e-02,1.384463e-02,1.351139e-02,
     &1.319438e-02,1.289246e-02,1.260456e-02/
      DATA (EXTICE2(I,27),I=1,43) /
C    BAND 27
     &3.417883e-01,2.113379e-01,1.526395e-01,1.193347e-01,9.790253e-02,
     &8.296715e-02,7.196979e-02,6.353806e-02,5.687024e-02,5.146670e-02,
     &4.700001e-02,4.324667e-02,4.004894e-02,3.729233e-02,3.489172e-02,
     &3.278257e-02,3.091499e-02,2.924987e-02,2.775609e-02,2.640859e-02,
     &2.518695e-02,2.407439e-02,2.305697e-02,2.212303e-02,2.126273e-02,
     &2.046774e-02,1.973090e-02,1.904610e-02,1.840801e-02,1.781204e-02,
     &1.725417e-02,1.673086e-02,1.623902e-02,1.577590e-02,1.533906e-02,
     &1.492634e-02,1.453580e-02,1.416571e-02,1.381450e-02,1.348078e-02,
     &1.316327e-02,1.286082e-02,1.257240e-02/
      DATA (EXTICE2(I,28),I=1,43) /
C    BAND 28
     &3.416111e-01,2.114124e-01,1.527734e-01,1.194809e-01,9.804612e-02,
     &8.310287e-02,7.209595e-02,6.365442e-02,5.697710e-02,5.156460e-02,
     &4.708957e-02,4.332850e-02,4.012361e-02,3.736037e-02,3.495364e-02,
     &3.283879e-02,3.096593e-02,2.929589e-02,2.779751e-02,2.644571e-02,
     &2.522004e-02,2.410369e-02,2.308271e-02,2.214542e-02,2.128195e-02,
     &2.048396e-02,1.974429e-02,1.905679e-02,1.841614e-02,1.781774e-02,
     &1.725754e-02,1.673203e-02,1.623807e-02,1.577293e-02,1.533416e-02,
     &1.491958e-02,1.452727e-02,1.415547e-02,1.380262e-02,1.346732e-02,
     &1.314830e-02,1.284439e-02,1.255456e-02/
      DATA (EXTICE2(I,29),I=1,43) /
C    BAND 29
     &4.196611e-01,2.493642e-01,1.761261e-01,1.357197e-01,1.102161e-01,
     &9.269376e-02,7.992985e-02,7.022538e-02,6.260168e-02,5.645603e-02,
     &5.139732e-02,4.716088e-02,4.356133e-02,4.046498e-02,3.777303e-02,
     &3.541094e-02,3.332137e-02,3.145954e-02,2.978998e-02,2.828419e-02,
     &2.691905e-02,2.567559e-02,2.453811e-02,2.349350e-02,2.253072e-02,
     &2.164042e-02,2.081464e-02,2.004652e-02,1.933015e-02,1.866041e-02,
     &1.803283e-02,1.744348e-02,1.688894e-02,1.636616e-02,1.587244e-02,
     &1.540539e-02,1.496287e-02,1.454295e-02,1.414392e-02,1.376423e-02,
     &1.340247e-02,1.305739e-02,1.272784e-02/
C     SINGLE-SCATTERING ALBEDO: Unitless
      DATA (SSAICE2(I,16),I=1,43) /
C    BAND 16
     &6.630615e-01,6.451169e-01,6.333696e-01,6.246927e-01,6.178420e-01,
     &6.121976e-01,6.074069e-01,6.032505e-01,5.995830e-01,5.963030e-01,
     &5.933372e-01,5.906311e-01,5.881427e-01,5.858395e-01,5.836955e-01,
     &5.816896e-01,5.798046e-01,5.780264e-01,5.763429e-01,5.747441e-01,
     &5.732213e-01,5.717672e-01,5.703754e-01,5.690403e-01,5.677571e-01,
     &5.665215e-01,5.653297e-01,5.641782e-01,5.630643e-01,5.619850e-01,
     &5.609381e-01,5.599214e-01,5.589328e-01,5.579707e-01,5.570333e-01,
     &5.561193e-01,5.552272e-01,5.543558e-01,5.535041e-01,5.526708e-01,
     &5.518551e-01,5.510561e-01,5.502729e-01/
      DATA (SSAICE2(I,17),I=1,43) /
C    BAND 17
     &7.689749e-01,7.398171e-01,7.205819e-01,7.065690e-01,6.956928e-01,
     &6.868989e-01,6.795813e-01,6.733606e-01,6.679838e-01,6.632742e-01,
     &6.591036e-01,6.553766e-01,6.520197e-01,6.489757e-01,6.461991e-01,
     &6.436531e-01,6.413075e-01,6.391375e-01,6.371221e-01,6.352438e-01,
     &6.334876e-01,6.318406e-01,6.302918e-01,6.288315e-01,6.274512e-01,
     &6.261436e-01,6.249022e-01,6.237211e-01,6.225953e-01,6.215201e-01,
     &6.204914e-01,6.195055e-01,6.185592e-01,6.176492e-01,6.167730e-01,
     &6.159280e-01,6.151120e-01,6.143228e-01,6.135587e-01,6.128177e-01,
     &6.120984e-01,6.113993e-01,6.107189e-01/
      DATA (SSAICE2(I,18),I=1,43) /
C    BAND 18
     &9.956167e-01,9.814770e-01,9.716104e-01,9.639746e-01,9.577179e-01,
     &9.524010e-01,9.477672e-01,9.436527e-01,9.399467e-01,9.365708e-01,
     &9.334672e-01,9.305921e-01,9.279118e-01,9.253993e-01,9.230330e-01,
     &9.207954e-01,9.186719e-01,9.166501e-01,9.147199e-01,9.128722e-01,
     &9.110997e-01,9.093956e-01,9.077544e-01,9.061708e-01,9.046406e-01,
     &9.031598e-01,9.017248e-01,9.003326e-01,8.989804e-01,8.976655e-01,
     &8.963857e-01,8.951389e-01,8.939233e-01,8.927370e-01,8.915785e-01,
     &8.904464e-01,8.893392e-01,8.882559e-01,8.871951e-01,8.861559e-01,
     &8.851373e-01,8.841383e-01,8.831581e-01/
      DATA (SSAICE2(I,19),I=1,43) /
C    BAND 19
     &9.723177e-01,9.452119e-01,9.267592e-01,9.127393e-01,9.014238e-01,
     &8.919334e-01,8.837584e-01,8.765773e-01,8.701736e-01,8.643950e-01,
     &8.591299e-01,8.542942e-01,8.498230e-01,8.456651e-01,8.417794e-01,
     &8.381324e-01,8.346964e-01,8.314484e-01,8.283687e-01,8.254408e-01,
     &8.226505e-01,8.199854e-01,8.174348e-01,8.149891e-01,8.126403e-01,
     &8.103808e-01,8.082041e-01,8.061044e-01,8.040765e-01,8.021156e-01,
     &8.002174e-01,7.983781e-01,7.965941e-01,7.948622e-01,7.931795e-01,
     &7.915432e-01,7.899508e-01,7.884002e-01,7.868891e-01,7.854156e-01,
     &7.839779e-01,7.825742e-01,7.812031e-01/
      DATA (SSAICE2(I,20),I=1,43) /
C    BAND 20
     &9.933294e-01,9.860917e-01,9.811564e-01,9.774008e-01,9.743652e-01,
     &9.718155e-01,9.696159e-01,9.676810e-01,9.659531e-01,9.643915e-01,
     &9.629667e-01,9.616561e-01,9.604426e-01,9.593125e-01,9.582548e-01,
     &9.572607e-01,9.563227e-01,9.554347e-01,9.545915e-01,9.537888e-01,
     &9.530226e-01,9.522898e-01,9.515874e-01,9.509130e-01,9.502643e-01,
     &9.496394e-01,9.490366e-01,9.484542e-01,9.478910e-01,9.473456e-01,
     &9.468169e-01,9.463039e-01,9.458056e-01,9.453212e-01,9.448499e-01,
     &9.443910e-01,9.439438e-01,9.435077e-01,9.430821e-01,9.426666e-01,
     &9.422607e-01,9.418638e-01,9.414756e-01/
      DATA (SSAICE2(I,21),I=1,43) /
C    BAND 21
     &9.900787e-01,9.828880e-01,9.779258e-01,9.741173e-01,9.710184e-01,
     &9.684012e-01,9.661332e-01,9.641301e-01,9.623352e-01,9.607083e-01,
     &9.592198e-01,9.578474e-01,9.565739e-01,9.553856e-01,9.542715e-01,
     &9.532226e-01,9.522314e-01,9.512919e-01,9.503986e-01,9.495472e-01,
     &9.487337e-01,9.479549e-01,9.472077e-01,9.464897e-01,9.457985e-01,
     &9.451322e-01,9.444890e-01,9.438673e-01,9.432656e-01,9.426826e-01,
     &9.421173e-01,9.415684e-01,9.410351e-01,9.405164e-01,9.400115e-01,
     &9.395198e-01,9.390404e-01,9.385728e-01,9.381164e-01,9.376707e-01,
     &9.372350e-01,9.368091e-01,9.363923e-01/
      DATA (SSAICE2(I,22),I=1,43) /
C    BAND 22
     &9.986793e-01,9.985239e-01,9.983911e-01,9.982715e-01,9.981606e-01,
     &9.980562e-01,9.979567e-01,9.978613e-01,9.977691e-01,9.976798e-01,
     &9.975929e-01,9.975081e-01,9.974251e-01,9.973438e-01,9.972640e-01,
     &9.971855e-01,9.971083e-01,9.970322e-01,9.969571e-01,9.968830e-01,
     &9.968099e-01,9.967375e-01,9.966660e-01,9.965951e-01,9.965250e-01,
     &9.964555e-01,9.963867e-01,9.963185e-01,9.962508e-01,9.961836e-01,
     &9.961170e-01,9.960508e-01,9.959851e-01,9.959198e-01,9.958550e-01,
     &9.957906e-01,9.957266e-01,9.956629e-01,9.955997e-01,9.955367e-01,
     &9.954742e-01,9.954119e-01,9.953500e-01/
      DATA (SSAICE2(I,23),I=1,43) /
C    BAND 23
     &9.997944e-01,9.997791e-01,9.997664e-01,9.997547e-01,9.997436e-01,
     &9.997327e-01,9.997219e-01,9.997110e-01,9.996999e-01,9.996886e-01,
     &9.996771e-01,9.996653e-01,9.996533e-01,9.996409e-01,9.996282e-01,
     &9.996152e-01,9.996019e-01,9.995883e-01,9.995743e-01,9.995599e-01,
     &9.995453e-01,9.995302e-01,9.995149e-01,9.994992e-01,9.994831e-01,
     &9.994667e-01,9.994500e-01,9.994329e-01,9.994154e-01,9.993976e-01,
     &9.993795e-01,9.993610e-01,9.993422e-01,9.993230e-01,9.993035e-01,
     &9.992837e-01,9.992635e-01,9.992429e-01,9.992221e-01,9.992008e-01,
     &9.991793e-01,9.991574e-01,9.991352e-01/
      DATA (SSAICE2(I,24),I=1,43) /
C    BAND 24
     &9.999949e-01,9.999947e-01,9.999943e-01,9.999939e-01,9.999934e-01,
     &9.999927e-01,9.999920e-01,9.999913e-01,9.999904e-01,9.999895e-01,
     &9.999885e-01,9.999874e-01,9.999863e-01,9.999851e-01,9.999838e-01,
     &9.999824e-01,9.999810e-01,9.999795e-01,9.999780e-01,9.999764e-01,
     &9.999747e-01,9.999729e-01,9.999711e-01,9.999692e-01,9.999673e-01,
     &9.999653e-01,9.999632e-01,9.999611e-01,9.999589e-01,9.999566e-01,
     &9.999543e-01,9.999519e-01,9.999495e-01,9.999470e-01,9.999444e-01,
     &9.999418e-01,9.999392e-01,9.999364e-01,9.999336e-01,9.999308e-01,
     &9.999279e-01,9.999249e-01,9.999219e-01/
      DATA (SSAICE2(I,25),I=1,43) /
C    BAND 25
     &9.999997e-01,9.999997e-01,9.999997e-01,9.999996e-01,9.999996e-01,
     &9.999995e-01,9.999994e-01,9.999993e-01,9.999993e-01,9.999992e-01,
     &9.999991e-01,9.999989e-01,9.999988e-01,9.999987e-01,9.999986e-01,
     &9.999984e-01,9.999983e-01,9.999981e-01,9.999980e-01,9.999978e-01,
     &9.999976e-01,9.999974e-01,9.999972e-01,9.999971e-01,9.999969e-01,
     &9.999966e-01,9.999964e-01,9.999962e-01,9.999960e-01,9.999957e-01,
     &9.999955e-01,9.999953e-01,9.999950e-01,9.999947e-01,9.999945e-01,
     &9.999942e-01,9.999939e-01,9.999936e-01,9.999934e-01,9.999931e-01,
     &9.999928e-01,9.999925e-01,9.999921e-01/
      DATA (SSAICE2(I,26),I=1,43) /
C    BAND 26
     &9.999997e-01,9.999996e-01,9.999996e-01,9.999995e-01,9.999994e-01,
     &9.999993e-01,9.999992e-01,9.999991e-01,9.999990e-01,9.999989e-01,
     &9.999987e-01,9.999986e-01,9.999984e-01,9.999982e-01,9.999980e-01,
     &9.999978e-01,9.999976e-01,9.999974e-01,9.999972e-01,9.999970e-01,
     &9.999967e-01,9.999965e-01,9.999962e-01,9.999959e-01,9.999956e-01,
     &9.999954e-01,9.999951e-01,9.999947e-01,9.999944e-01,9.999941e-01,
     &9.999938e-01,9.999934e-01,9.999931e-01,9.999927e-01,9.999923e-01,
     &9.999920e-01,9.999916e-01,9.999912e-01,9.999908e-01,9.999904e-01,
     &9.999899e-01,9.999895e-01,9.999891e-01/
      DATA (SSAICE2(I,27),I=1,43) /
C    BAND 27
     &9.999987e-01,9.999987e-01,9.999985e-01,9.999984e-01,9.999982e-01,
     &9.999980e-01,9.999978e-01,9.999976e-01,9.999973e-01,9.999970e-01,
     &9.999967e-01,9.999964e-01,9.999960e-01,9.999956e-01,9.999952e-01,
     &9.999948e-01,9.999944e-01,9.999939e-01,9.999934e-01,9.999929e-01,
     &9.999924e-01,9.999918e-01,9.999913e-01,9.999907e-01,9.999901e-01,
     &9.999894e-01,9.999888e-01,9.999881e-01,9.999874e-01,9.999867e-01,
     &9.999860e-01,9.999853e-01,9.999845e-01,9.999837e-01,9.999829e-01,
     &9.999821e-01,9.999813e-01,9.999804e-01,9.999796e-01,9.999787e-01,
     &9.999778e-01,9.999768e-01,9.999759e-01/
      DATA (SSAICE2(I,28),I=1,43) /
C    BAND 28
     &9.999989e-01,9.999989e-01,9.999987e-01,9.999986e-01,9.999984e-01,
     &9.999982e-01,9.999980e-01,9.999978e-01,9.999975e-01,9.999972e-01,
     &9.999969e-01,9.999966e-01,9.999962e-01,9.999958e-01,9.999954e-01,
     &9.999950e-01,9.999945e-01,9.999941e-01,9.999936e-01,9.999931e-01,
     &9.999925e-01,9.999920e-01,9.999914e-01,9.999908e-01,9.999902e-01,
     &9.999896e-01,9.999889e-01,9.999883e-01,9.999876e-01,9.999869e-01,
     &9.999861e-01,9.999854e-01,9.999846e-01,9.999838e-01,9.999830e-01,
     &9.999822e-01,9.999814e-01,9.999805e-01,9.999796e-01,9.999787e-01,
     &9.999778e-01,9.999769e-01,9.999759e-01/
      DATA (SSAICE2(I,29),I=1,43) /
C    BAND 29
     &7.042143e-01,6.691161e-01,6.463240e-01,6.296590e-01,6.166381e-01,
     &6.060183e-01,5.970908e-01,5.894144e-01,5.826968e-01,5.767343e-01,
     &5.713804e-01,5.665256e-01,5.620867e-01,5.579987e-01,5.542101e-01,
     &5.506794e-01,5.473727e-01,5.442620e-01,5.413239e-01,5.385389e-01,
     &5.358901e-01,5.333633e-01,5.309460e-01,5.286277e-01,5.263988e-01,
     &5.242512e-01,5.221777e-01,5.201719e-01,5.182280e-01,5.163410e-01,
     &5.145062e-01,5.127197e-01,5.109776e-01,5.092766e-01,5.076137e-01,
     &5.059860e-01,5.043911e-01,5.028266e-01,5.012904e-01,4.997805e-01,
     &4.982951e-01,4.968326e-01,4.953913e-01/
C     ASYMMETRY FACTOR: Unitless
      DATA (ASYICE2(I,16),I=1,43) /
C    BAND 16
     &7.946655e-01,8.547685e-01,8.806016e-01,8.949880e-01,9.041676e-01,
     &9.105399e-01,9.152249e-01,9.188160e-01,9.216573e-01,9.239620e-01,
     &9.258695e-01,9.274745e-01,9.288441e-01,9.300267e-01,9.310584e-01,
     &9.319665e-01,9.327721e-01,9.334918e-01,9.341387e-01,9.347236e-01,
     &9.352551e-01,9.357402e-01,9.361850e-01,9.365942e-01,9.369722e-01,
     &9.373225e-01,9.376481e-01,9.379516e-01,9.382352e-01,9.385010e-01,
     &9.387505e-01,9.389854e-01,9.392070e-01,9.394163e-01,9.396145e-01,
     &9.398024e-01,9.399809e-01,9.401508e-01,9.403126e-01,9.404670e-01,
     &9.406144e-01,9.407555e-01,9.408906e-01/
      DATA (ASYICE2(I,17),I=1,43) /
C    BAND 17
     &9.078091e-01,9.195850e-01,9.267250e-01,9.317083e-01,9.354632e-01,
     &9.384323e-01,9.408597e-01,9.428935e-01,9.446301e-01,9.461351e-01,
     &9.474555e-01,9.486259e-01,9.496722e-01,9.506146e-01,9.514688e-01,
     &9.522476e-01,9.529612e-01,9.536181e-01,9.542251e-01,9.547883e-01,
     &9.553124e-01,9.558019e-01,9.562601e-01,9.566904e-01,9.570953e-01,
     &9.574773e-01,9.578385e-01,9.581806e-01,9.585054e-01,9.588142e-01,
     &9.591083e-01,9.593888e-01,9.596569e-01,9.599135e-01,9.601593e-01,
     &9.603952e-01,9.606219e-01,9.608399e-01,9.610499e-01,9.612523e-01,
     &9.614477e-01,9.616365e-01,9.618192e-01/
      DATA (ASYICE2(I,18),I=1,43) /
C    BAND 18
     &8.322045e-01,8.528693e-01,8.648167e-01,8.729163e-01,8.789054e-01,
     &8.835845e-01,8.873819e-01,8.905511e-01,8.932532e-01,8.955965e-01,
     &8.976567e-01,8.994887e-01,9.011334e-01,9.026221e-01,9.039791e-01,
     &9.052237e-01,9.063715e-01,9.074349e-01,9.084245e-01,9.093489e-01,
     &9.102154e-01,9.110303e-01,9.117987e-01,9.125253e-01,9.132140e-01,
     &9.138682e-01,9.144910e-01,9.150850e-01,9.156524e-01,9.161955e-01,
     &9.167160e-01,9.172157e-01,9.176959e-01,9.181581e-01,9.186034e-01,
     &9.190330e-01,9.194478e-01,9.198488e-01,9.202368e-01,9.206126e-01,
     &9.209768e-01,9.213301e-01,9.216731e-01/
      DATA (ASYICE2(I,19),I=1,43) /
C    BAND 19
     &8.116560e-01,8.488278e-01,8.674331e-01,8.788148e-01,8.865810e-01,
     &8.922595e-01,8.966149e-01,9.000747e-01,9.028980e-01,9.052513e-01,
     &9.072468e-01,9.089632e-01,9.104574e-01,9.117713e-01,9.129371e-01,
     &9.139793e-01,9.149174e-01,9.157668e-01,9.165400e-01,9.172473e-01,
     &9.178970e-01,9.184962e-01,9.190508e-01,9.195658e-01,9.200455e-01,
     &9.204935e-01,9.209130e-01,9.213067e-01,9.216771e-01,9.220262e-01,
     &9.223560e-01,9.226680e-01,9.229636e-01,9.232443e-01,9.235112e-01,
     &9.237652e-01,9.240074e-01,9.242385e-01,9.244594e-01,9.246708e-01,
     &9.248733e-01,9.250674e-01,9.252536e-01/
      DATA (ASYICE2(I,20),I=1,43) /
C    BAND 20
     &8.047113e-01,8.402864e-01,8.570332e-01,8.668455e-01,8.733206e-01,
     &8.779272e-01,8.813796e-01,8.840676e-01,8.862225e-01,8.879904e-01,
     &8.894682e-01,8.907228e-01,8.918019e-01,8.927404e-01,8.935645e-01,
     &8.942943e-01,8.949452e-01,8.955296e-01,8.960574e-01,8.965366e-01,
     &8.969736e-01,8.973740e-01,8.977422e-01,8.980820e-01,8.983966e-01,
     &8.986889e-01,8.989611e-01,8.992153e-01,8.994533e-01,8.996766e-01,
     &8.998865e-01,9.000843e-01,9.002709e-01,9.004474e-01,9.006146e-01,
     &9.007731e-01,9.009237e-01,9.010670e-01,9.012034e-01,9.013336e-01,
     &9.014579e-01,9.015767e-01,9.016904e-01/
      DATA (ASYICE2(I,21),I=1,43) /
C    BAND 21
     &8.179122e-01,8.480726e-01,8.621945e-01,8.704354e-01,8.758555e-01,
     &8.797007e-01,8.825750e-01,8.848078e-01,8.865939e-01,8.880564e-01,
     &8.892765e-01,8.903105e-01,8.911982e-01,8.919689e-01,8.926446e-01,
     &8.932419e-01,8.937738e-01,8.942506e-01,8.946806e-01,8.950702e-01,
     &8.954251e-01,8.957497e-01,8.960477e-01,8.963223e-01,8.965762e-01,
     &8.968116e-01,8.970306e-01,8.972347e-01,8.974255e-01,8.976042e-01,
     &8.977720e-01,8.979298e-01,8.980784e-01,8.982188e-01,8.983515e-01,
     &8.984771e-01,8.985963e-01,8.987095e-01,8.988171e-01,8.989195e-01,
     &8.990172e-01,8.991104e-01,8.991994e-01/
      DATA (ASYICE2(I,22),I=1,43) /
C    BAND 22
     &8.169789e-01,8.455024e-01,8.586925e-01,8.663283e-01,8.713217e-01,
     &8.748488e-01,8.774765e-01,8.795122e-01,8.811370e-01,8.824649e-01,
     &8.835711e-01,8.845073e-01,8.853103e-01,8.860068e-01,8.866170e-01,
     &8.871560e-01,8.876358e-01,8.880658e-01,8.884533e-01,8.888044e-01,
     &8.891242e-01,8.894166e-01,8.896851e-01,8.899324e-01,8.901612e-01,
     &8.903733e-01,8.905706e-01,8.907545e-01,8.909265e-01,8.910876e-01,
     &8.912388e-01,8.913812e-01,8.915153e-01,8.916419e-01,8.917617e-01,
     &8.918752e-01,8.919829e-01,8.920851e-01,8.921824e-01,8.922751e-01,
     &8.923635e-01,8.924478e-01,8.925284e-01/
      DATA (ASYICE2(I,23),I=1,43) /
C    BAND 23
     &8.387642e-01,8.569979e-01,8.658630e-01,8.711825e-01,8.747605e-01,
     &8.773472e-01,8.793129e-01,8.808621e-01,8.821179e-01,8.831583e-01,
     &8.840361e-01,8.847875e-01,8.854388e-01,8.860094e-01,8.865138e-01,
     &8.869634e-01,8.873668e-01,8.877310e-01,8.880617e-01,8.883635e-01,
     &8.886401e-01,8.888947e-01,8.891298e-01,8.893477e-01,8.895504e-01,
     &8.897393e-01,8.899159e-01,8.900815e-01,8.902370e-01,8.903833e-01,
     &8.905214e-01,8.906518e-01,8.907753e-01,8.908924e-01,8.910036e-01,
     &8.911094e-01,8.912101e-01,8.913062e-01,8.913979e-01,8.914856e-01,
     &8.915695e-01,8.916498e-01,8.917269e-01/
      DATA (ASYICE2(I,24),I=1,43) /
C    BAND 24
     &8.522208e-01,8.648132e-01,8.711224e-01,8.749901e-01,8.776354e-01,
     &8.795743e-01,8.810649e-01,8.822518e-01,8.832225e-01,8.840333e-01,
     &8.847224e-01,8.853162e-01,8.858342e-01,8.862906e-01,8.866962e-01,
     &8.870595e-01,8.873871e-01,8.876842e-01,8.879551e-01,8.882032e-01,
     &8.884316e-01,8.886425e-01,8.888380e-01,8.890199e-01,8.891895e-01,
     &8.893481e-01,8.894968e-01,8.896366e-01,8.897683e-01,8.898926e-01,
     &8.900102e-01,8.901215e-01,8.902272e-01,8.903276e-01,8.904232e-01,
     &8.905144e-01,8.906014e-01,8.906845e-01,8.907640e-01,8.908402e-01,
     &8.909132e-01,8.909834e-01,8.910507e-01/
      DATA (ASYICE2(I,25),I=1,43) /
C    BAND 25
     &8.578202e-01,8.683033e-01,8.735431e-01,8.767488e-01,8.789378e-01,
     &8.805399e-01,8.817701e-01,8.827485e-01,8.835480e-01,8.842152e-01,
     &8.847817e-01,8.852696e-01,8.856949e-01,8.860694e-01,8.864020e-01,
     &8.866997e-01,8.869681e-01,8.872113e-01,8.874330e-01,8.876360e-01,
     &8.878227e-01,8.879951e-01,8.881548e-01,8.883033e-01,8.884418e-01,
     &8.885712e-01,8.886926e-01,8.888066e-01,8.889139e-01,8.890152e-01,
     &8.891110e-01,8.892017e-01,8.892877e-01,8.893695e-01,8.894473e-01,
     &8.895214e-01,8.895921e-01,8.896597e-01,8.897243e-01,8.897862e-01,
     &8.898456e-01,8.899025e-01,8.899572e-01/
      DATA (ASYICE2(I,26),I=1,43) /
C    BAND 26
     &8.625615e-01,8.713831e-01,8.755799e-01,8.780560e-01,8.796983e-01,
     &8.808714e-01,8.817534e-01,8.824420e-01,8.829953e-01,8.834501e-01,
     &8.838310e-01,8.841549e-01,8.844338e-01,8.846767e-01,8.848902e-01,
     &8.850795e-01,8.852484e-01,8.854002e-01,8.855374e-01,8.856620e-01,
     &8.857758e-01,8.858800e-01,8.859759e-01,8.860644e-01,8.861464e-01,
     &8.862225e-01,8.862935e-01,8.863598e-01,8.864218e-01,8.864800e-01,
     &8.865347e-01,8.865863e-01,8.866349e-01,8.866809e-01,8.867245e-01,
     &8.867658e-01,8.868050e-01,8.868423e-01,8.868778e-01,8.869117e-01,
     &8.869440e-01,8.869749e-01,8.870044e-01/
      DATA (ASYICE2(I,27),I=1,43) /
C    BAND 27
     &8.587495e-01,8.684764e-01,8.728189e-01,8.752872e-01,8.768846e-01,
     &8.780060e-01,8.788386e-01,8.794824e-01,8.799960e-01,8.804159e-01,
     &8.807660e-01,8.810626e-01,8.813175e-01,8.815390e-01,8.817335e-01,
     &8.819057e-01,8.820593e-01,8.821973e-01,8.823220e-01,8.824353e-01,
     &8.825387e-01,8.826336e-01,8.827209e-01,8.828016e-01,8.828764e-01,
     &8.829459e-01,8.830108e-01,8.830715e-01,8.831283e-01,8.831817e-01,
     &8.832320e-01,8.832795e-01,8.833244e-01,8.833668e-01,8.834071e-01,
     &8.834454e-01,8.834817e-01,8.835164e-01,8.835495e-01,8.835811e-01,
     &8.836113e-01,8.836402e-01,8.836679e-01/
      DATA (ASYICE2(I,28),I=1,43) /
C    BAND 28
     &8.561110e-01,8.678583e-01,8.727554e-01,8.753892e-01,8.770154e-01,
     &8.781109e-01,8.788949e-01,8.794812e-01,8.799348e-01,8.802952e-01,
     &8.805880e-01,8.808300e-01,8.810331e-01,8.812058e-01,8.813543e-01,
     &8.814832e-01,8.815960e-01,8.816956e-01,8.817839e-01,8.818629e-01,
     &8.819339e-01,8.819979e-01,8.820560e-01,8.821089e-01,8.821573e-01,
     &8.822016e-01,8.822425e-01,8.822801e-01,8.823150e-01,8.823474e-01,
     &8.823775e-01,8.824056e-01,8.824318e-01,8.824564e-01,8.824795e-01,
     &8.825011e-01,8.825215e-01,8.825408e-01,8.825589e-01,8.825761e-01,
     &8.825924e-01,8.826078e-01,8.826224e-01/
      DATA (ASYICE2(I,29),I=1,43) /
C    BAND 29
     &8.311124e-01,8.688197e-01,8.900274e-01,9.040696e-01,9.142334e-01,
     &9.220181e-01,9.282195e-01,9.333048e-01,9.375689e-01,9.412085e-01,
     &9.443604e-01,9.471230e-01,9.495694e-01,9.517549e-01,9.537224e-01,
     &9.555057e-01,9.571316e-01,9.586222e-01,9.599952e-01,9.612656e-01,
     &9.624458e-01,9.635461e-01,9.645756e-01,9.655418e-01,9.664513e-01,
     &9.673098e-01,9.681222e-01,9.688928e-01,9.696256e-01,9.703237e-01,
     &9.709903e-01,9.716280e-01,9.722391e-01,9.728258e-01,9.733901e-01,
     &9.739336e-01,9.744579e-01,9.749645e-01,9.754546e-01,9.759294e-01,
     &9.763901e-01,9.768376e-01,9.772727e-01/
C     Hexagonal Ice Particle Parameterization
C     EXTINCTION UNITS (EXT COEF/IWC): [(m^-1)/(g m^-3)]
      DATA (EXTICE3(I,16),I=1,46) /
C    BAND 16
     &5.194013e-01,3.215089e-01,2.327917e-01,1.824424e-01,1.499977e-01,
     &1.273492e-01,1.106421e-01,9.780982e-02,8.764435e-02,7.939266e-02,
     &7.256081e-02,6.681137e-02,6.190600e-02,5.767154e-02,5.397915e-02,
     &5.073102e-02,4.785151e-02,4.528125e-02,4.297296e-02,4.088853e-02,
     &3.899690e-02,3.727251e-02,3.569411e-02,3.424393e-02,3.290694e-02,
     &3.167040e-02,3.052340e-02,2.945654e-02,2.846172e-02,2.753188e-02,
     &2.666085e-02,2.584322e-02,2.507423e-02,2.434967e-02,2.366579e-02,
     &2.301926e-02,2.240711e-02,2.182666e-02,2.127551e-02,2.075150e-02,
     &2.025267e-02,1.977725e-02,1.932364e-02,1.889035e-02,1.847607e-02,
     &1.807956e-02/
      DATA (EXTICE3(I,17),I=1,46) /
C    BAND 17
     &4.901155e-01,3.065286e-01,2.230800e-01,1.753951e-01,1.445402e-01,
     &1.229417e-01,1.069777e-01,9.469760e-02,8.495824e-02,7.704501e-02,
     &7.048834e-02,6.496693e-02,6.025353e-02,5.618286e-02,5.263186e-02,
     &4.950698e-02,4.673585e-02,4.426164e-02,4.203904e-02,4.003153e-02,
     &3.820932e-02,3.654790e-02,3.502688e-02,3.362919e-02,3.234041e-02,
     &3.114829e-02,3.004234e-02,2.901356e-02,2.805413e-02,2.715727e-02,
     &2.631705e-02,2.552828e-02,2.478637e-02,2.408725e-02,2.342734e-02,
     &2.280343e-02,2.221264e-02,2.165242e-02,2.112043e-02,2.061461e-02,
     &2.013308e-02,1.967411e-02,1.923616e-02,1.881783e-02,1.841781e-02,
     &1.803494e-02/
      DATA (EXTICE3(I,18),I=1,46) /
C    BAND 18
     &5.056264e-01,3.160261e-01,2.298442e-01,1.805973e-01,1.487318e-01,
     &1.264258e-01,1.099389e-01,9.725656e-02,8.719819e-02,7.902576e-02,
     &7.225433e-02,6.655206e-02,6.168427e-02,5.748028e-02,5.381296e-02,
     &5.058572e-02,4.772383e-02,4.516857e-02,4.287317e-02,4.079990e-02,
     &3.891801e-02,3.720217e-02,3.563133e-02,3.418786e-02,3.285686e-02,
     &3.162569e-02,3.048352e-02,2.942104e-02,2.843018e-02,2.750395e-02,
     &2.663621e-02,2.582160e-02,2.505539e-02,2.433337e-02,2.365185e-02,
     &2.300750e-02,2.239736e-02,2.181878e-02,2.126937e-02,2.074699e-02,
     &2.024968e-02,1.977567e-02,1.932338e-02,1.889134e-02,1.847823e-02,
     &1.808281e-02/
      DATA (EXTICE3(I,19),I=1,46) /
C    BAND 19
     &4.881605e-01,3.055237e-01,2.225070e-01,1.750688e-01,1.443736e-01,
     &1.228869e-01,1.070054e-01,9.478893e-02,8.509997e-02,7.722769e-02,
     &7.070495e-02,6.521211e-02,6.052311e-02,5.647351e-02,5.294088e-02,
     &4.983217e-02,4.707539e-02,4.461398e-02,4.240288e-02,4.040575e-02,
     &3.859298e-02,3.694016e-02,3.542701e-02,3.403655e-02,3.275444e-02,
     &3.156849e-02,3.046827e-02,2.944481e-02,2.849034e-02,2.759812e-02,
     &2.676226e-02,2.597757e-02,2.523949e-02,2.454400e-02,2.388750e-02,
     &2.326682e-02,2.267909e-02,2.212176e-02,2.159253e-02,2.108933e-02,
     &2.061028e-02,2.015369e-02,1.971801e-02,1.930184e-02,1.890389e-02,
     &1.852300e-02/
      DATA (EXTICE3(I,20),I=1,46) /
C    BAND 20
     &5.103703e-01,3.188144e-01,2.317435e-01,1.819887e-01,1.497944e-01,
     &1.272584e-01,1.106013e-01,9.778822e-02,8.762610e-02,7.936938e-02,
     &7.252809e-02,6.676701e-02,6.184901e-02,5.760165e-02,5.389651e-02,
     &5.063598e-02,4.774457e-02,4.516295e-02,4.284387e-02,4.074922e-02,
     &3.884792e-02,3.711438e-02,3.552734e-02,3.406898e-02,3.272425e-02,
     &3.148038e-02,3.032643e-02,2.925299e-02,2.825191e-02,2.731612e-02,
     &2.643943e-02,2.561642e-02,2.484230e-02,2.411284e-02,2.342429e-02,
     &2.277329e-02,2.215686e-02,2.157231e-02,2.101724e-02,2.048946e-02,
     &1.998702e-02,1.950813e-02,1.905118e-02,1.861468e-02,1.819730e-02,
     &1.779781e-02/
      DATA (EXTICE3(I,21),I=1,46) /
C    BAND 21
     &5.031161e-01,3.144511e-01,2.286942e-01,1.796903e-01,1.479819e-01,
     &1.257860e-01,1.093803e-01,9.676059e-02,8.675183e-02,7.861971e-02,
     &7.188168e-02,6.620754e-02,6.136376e-02,5.718050e-02,5.353127e-02,
     &5.031995e-02,4.747218e-02,4.492952e-02,4.264544e-02,4.058240e-02,
     &3.870979e-02,3.700242e-02,3.543933e-02,3.400297e-02,3.267854e-02,
     &3.145345e-02,3.031691e-02,2.925967e-02,2.827370e-02,2.735203e-02,
     &2.648858e-02,2.567798e-02,2.491555e-02,2.419710e-02,2.351893e-02,
     &2.287776e-02,2.227063e-02,2.169491e-02,2.114821e-02,2.062840e-02,
     &2.013354e-02,1.966188e-02,1.921182e-02,1.878191e-02,1.837083e-02,
     &1.797737e-02/
      DATA (EXTICE3(I,22),I=1,46) /
C    BAND 22
     &4.949453e-01,3.095918e-01,2.253402e-01,1.771964e-01,1.460446e-01,
     &1.242383e-01,1.081206e-01,9.572235e-02,8.588928e-02,7.789990e-02,
     &7.128013e-02,6.570559e-02,6.094684e-02,5.683701e-02,5.325183e-02,
     &5.009688e-02,4.729909e-02,4.480106e-02,4.255708e-02,4.053025e-02,
     &3.869051e-02,3.701310e-02,3.547745e-02,3.406631e-02,3.276512e-02,
     &3.156153e-02,3.044494e-02,2.940626e-02,2.843759e-02,2.753211e-02,
     &2.668381e-02,2.588744e-02,2.513839e-02,2.443255e-02,2.376629e-02,
     &2.313637e-02,2.253990e-02,2.197428e-02,2.143718e-02,2.092649e-02,
     &2.044032e-02,1.997694e-02,1.953478e-02,1.911241e-02,1.870855e-02,
     &1.832199e-02/
      DATA (EXTICE3(I,23),I=1,46) /
C    BAND 23
     &5.052816e-01,3.157665e-01,2.296233e-01,1.803986e-01,1.485473e-01,
     &1.262514e-01,1.097718e-01,9.709524e-02,8.704139e-02,7.887264e-02,
     &7.210424e-02,6.640454e-02,6.153894e-02,5.733683e-02,5.367116e-02,
     &5.044537e-02,4.758477e-02,4.503066e-02,4.273629e-02,4.066395e-02,
     &3.878291e-02,3.706784e-02,3.549771e-02,3.405488e-02,3.272448e-02,
     &3.149387e-02,3.035221e-02,2.929020e-02,2.829979e-02,2.737397e-02,
     &2.650663e-02,2.569238e-02,2.492651e-02,2.420482e-02,2.352361e-02,
     &2.287954e-02,2.226968e-02,2.169136e-02,2.114220e-02,2.062005e-02,
     &2.012296e-02,1.964917e-02,1.919709e-02,1.876524e-02,1.835231e-02,
     &1.795707e-02/
      DATA (EXTICE3(I,24),I=1,46) /
C    BAND 24
     &5.042067e-01,3.151195e-01,2.291708e-01,1.800573e-01,1.482779e-01,
     &1.260324e-01,1.095900e-01,9.694202e-02,8.691087e-02,7.876056e-02,
     &7.200745e-02,6.632062e-02,6.146600e-02,5.727338e-02,5.361599e-02,
     &5.039749e-02,4.754334e-02,4.499500e-02,4.270580e-02,4.063815e-02,
     &3.876135e-02,3.705016e-02,3.548357e-02,3.404400e-02,3.271661e-02,
     &3.148877e-02,3.034969e-02,2.929008e-02,2.830191e-02,2.737818e-02,
     &2.651279e-02,2.570039e-02,2.493624e-02,2.421618e-02,2.353650e-02,
     &2.289390e-02,2.228541e-02,2.170840e-02,2.116048e-02,2.063950e-02,
     &2.014354e-02,1.967082e-02,1.921975e-02,1.878888e-02,1.837688e-02,
     &1.798254e-02/
      DATA (EXTICE3(I,25),I=1,46) /
C    BAND 25
     &5.022507e-01,3.139246e-01,2.283218e-01,1.794059e-01,1.477544e-01,
     &1.255984e-01,1.092222e-01,9.662516e-02,8.663439e-02,7.851688e-02,
     &7.179095e-02,6.612700e-02,6.129193e-02,5.711618e-02,5.347351e-02,
     &5.026796e-02,4.742530e-02,4.488721e-02,4.260724e-02,4.054790e-02,
     &3.867866e-02,3.697435e-02,3.541407e-02,3.398029e-02,3.265824e-02,
     &3.143535e-02,3.030085e-02,2.924551e-02,2.826131e-02,2.734130e-02,
     &2.647939e-02,2.567026e-02,2.490919e-02,2.419203e-02,2.351509e-02,
     &2.287507e-02,2.226903e-02,2.169434e-02,2.114862e-02,2.062975e-02,
     &2.013578e-02,1.966496e-02,1.921571e-02,1.878658e-02,1.837623e-02,
     &1.798348e-02/
      DATA (EXTICE3(I,26),I=1,46) /
C    BAND 26
     &5.068316e-01,3.166869e-01,2.302576e-01,1.808693e-01,1.489122e-01,
     &1.265423e-01,1.100080e-01,9.728926e-02,8.720201e-02,7.900612e-02,
     &7.221524e-02,6.649660e-02,6.161484e-02,5.739877e-02,5.372093e-02,
     &5.048442e-02,4.761431e-02,4.505172e-02,4.274972e-02,4.067050e-02,
     &3.878321e-02,3.706244e-02,3.548710e-02,3.403948e-02,3.270466e-02,
     &3.146995e-02,3.032450e-02,2.925897e-02,2.826527e-02,2.733638e-02,
     &2.646615e-02,2.564920e-02,2.488078e-02,2.415670e-02,2.347322e-02,
     &2.282702e-02,2.221513e-02,2.163489e-02,2.108390e-02,2.056002e-02,
     &2.006128e-02,1.958591e-02,1.913232e-02,1.869904e-02,1.828474e-02,
     &1.788819e-02/
      DATA (EXTICE3(I,27),I=1,46) /
C    BAND 27
     &5.077707e-01,3.172636e-01,2.306695e-01,1.811871e-01,1.491691e-01,
     &1.267565e-01,1.101907e-01,9.744773e-02,8.734125e-02,7.912973e-02,
     &7.232591e-02,6.659637e-02,6.170530e-02,5.748120e-02,5.379634e-02,
     &5.055367e-02,4.767809e-02,4.511061e-02,4.280423e-02,4.072104e-02,
     &3.883015e-02,3.710611e-02,3.552776e-02,3.407738e-02,3.274002e-02,
     &3.150296e-02,3.035532e-02,2.928776e-02,2.829216e-02,2.736150e-02,
     &2.648961e-02,2.567111e-02,2.490123e-02,2.417576e-02,2.349098e-02,
     &2.284354e-02,2.223049e-02,2.164914e-02,2.109711e-02,2.057222e-02,
     &2.007253e-02,1.959626e-02,1.914181e-02,1.870770e-02,1.829261e-02,
     &1.789531e-02/
      DATA (EXTICE3(I,28),I=1,46) /
C    BAND 28
     &5.062281e-01,3.163402e-01,2.300275e-01,1.807060e-01,1.487921e-01,
     &1.264523e-01,1.099403e-01,9.723879e-02,8.716516e-02,7.898034e-02,
     &7.219863e-02,6.648771e-02,6.161254e-02,5.740217e-02,5.372929e-02,
     &5.049716e-02,4.763092e-02,4.507179e-02,4.277290e-02,4.069649e-02,
     &3.881175e-02,3.709331e-02,3.552008e-02,3.407442e-02,3.274141e-02,
     &3.150837e-02,3.036447e-02,2.930037e-02,2.830801e-02,2.738037e-02,
     &2.651132e-02,2.569547e-02,2.492810e-02,2.420499e-02,2.352243e-02,
     &2.287710e-02,2.226604e-02,2.168658e-02,2.113634e-02,2.061316e-02,
     &2.011510e-02,1.964038e-02,1.918740e-02,1.875471e-02,1.834096e-02,
     &1.794495e-02/
      DATA (EXTICE3(I,29),I=1,46) /
C    BAND 29
     &1.338834e-01,1.924912e-01,1.755523e-01,1.534793e-01,1.343937e-01,
     &1.187883e-01,1.060654e-01,9.559106e-02,8.685880e-02,7.948698e-02,
     &7.319086e-02,6.775669e-02,6.302215e-02,5.886236e-02,5.517996e-02,
     &5.189810e-02,4.895539e-02,4.630225e-02,4.389823e-02,4.171002e-02,
     &3.970998e-02,3.787493e-02,3.618537e-02,3.462471e-02,3.317880e-02,
     &3.183547e-02,3.058421e-02,2.941590e-02,2.832256e-02,2.729724e-02,
     &2.633377e-02,2.542675e-02,2.457136e-02,2.376332e-02,2.299882e-02,
     &2.227443e-02,2.158707e-02,2.093400e-02,2.031270e-02,1.972091e-02,
     &1.915659e-02,1.861787e-02,1.810304e-02,1.761055e-02,1.713899e-02,
     &1.668704e-02/
C     SINGLE-SCATTERING ALBEDO: Unitless
      DATA (SSAICE3(I,16),I=1,46) /
C    BAND 16
     &6.749442e-01,6.649947e-01,6.565828e-01,6.489928e-01,6.420046e-01,
     &6.355231e-01,6.294964e-01,6.238901e-01,6.186783e-01,6.138395e-01,
     &6.093543e-01,6.052049e-01,6.013742e-01,5.978457e-01,5.946030e-01,
     &5.916302e-01,5.889115e-01,5.864310e-01,5.841731e-01,5.821221e-01,
     &5.802624e-01,5.785785e-01,5.770549e-01,5.756759e-01,5.744262e-01,
     &5.732901e-01,5.722524e-01,5.712974e-01,5.704097e-01,5.695739e-01,
     &5.687747e-01,5.679964e-01,5.672238e-01,5.664415e-01,5.656340e-01,
     &5.647860e-01,5.638821e-01,5.629070e-01,5.618452e-01,5.606815e-01,
     &5.594006e-01,5.579870e-01,5.564255e-01,5.547008e-01,5.527976e-01,
     &5.507005e-01/
      DATA (SSAICE3(I,17),I=1,46) /
C    BAND 17
     &7.628550e-01,7.567297e-01,7.508463e-01,7.451972e-01,7.397745e-01,
     &7.345705e-01,7.295775e-01,7.247881e-01,7.201945e-01,7.157894e-01,
     &7.115652e-01,7.075145e-01,7.036300e-01,6.999044e-01,6.963304e-01,
     &6.929007e-01,6.896083e-01,6.864460e-01,6.834067e-01,6.804833e-01,
     &6.776690e-01,6.749567e-01,6.723397e-01,6.698109e-01,6.673637e-01,
     &6.649913e-01,6.626870e-01,6.604441e-01,6.582561e-01,6.561163e-01,
     &6.540182e-01,6.519554e-01,6.499215e-01,6.479099e-01,6.459145e-01,
     &6.439289e-01,6.419468e-01,6.399621e-01,6.379686e-01,6.359601e-01,
     &6.339306e-01,6.318740e-01,6.297845e-01,6.276559e-01,6.254825e-01,
     &6.232583e-01/
      DATA (SSAICE3(I,18),I=1,46) /
C    BAND 18
     &9.924147e-01,9.882792e-01,9.842257e-01,9.802522e-01,9.763566e-01,
     &9.725367e-01,9.687905e-01,9.651157e-01,9.615104e-01,9.579725e-01,
     &9.544997e-01,9.510901e-01,9.477416e-01,9.444520e-01,9.412194e-01,
     &9.380415e-01,9.349165e-01,9.318421e-01,9.288164e-01,9.258373e-01,
     &9.229027e-01,9.200106e-01,9.171589e-01,9.143457e-01,9.115688e-01,
     &9.088263e-01,9.061161e-01,9.034362e-01,9.007846e-01,8.981592e-01,
     &8.955581e-01,8.929792e-01,8.904206e-01,8.878803e-01,8.853562e-01,
     &8.828464e-01,8.803488e-01,8.778616e-01,8.753827e-01,8.729102e-01,
     &8.704421e-01,8.679764e-01,8.655112e-01,8.630445e-01,8.605744e-01,
     &8.580989e-01/
      DATA (SSAICE3(I,19),I=1,46) /
C    BAND 19
     &9.629413e-01,9.517182e-01,9.409209e-01,9.305366e-01,9.205529e-01,
     &9.109569e-01,9.017362e-01,8.928780e-01,8.843699e-01,8.761992e-01,
     &8.683536e-01,8.608204e-01,8.535873e-01,8.466417e-01,8.399712e-01,
     &8.335635e-01,8.274062e-01,8.214868e-01,8.157932e-01,8.103129e-01,
     &8.050336e-01,7.999432e-01,7.950294e-01,7.902798e-01,7.856825e-01,
     &7.812250e-01,7.768954e-01,7.726815e-01,7.685711e-01,7.645522e-01,
     &7.606126e-01,7.567404e-01,7.529234e-01,7.491498e-01,7.454074e-01,
     &7.416844e-01,7.379688e-01,7.342485e-01,7.305118e-01,7.267468e-01,
     &7.229415e-01,7.190841e-01,7.151628e-01,7.111657e-01,7.070811e-01,
     &7.028972e-01/
      DATA (SSAICE3(I,20),I=1,46) /
C    BAND 20
     &9.942270e-01,9.909206e-01,9.876775e-01,9.844960e-01,9.813746e-01,
     &9.783114e-01,9.753049e-01,9.723535e-01,9.694553e-01,9.666088e-01,
     &9.638123e-01,9.610641e-01,9.583626e-01,9.557060e-01,9.530928e-01,
     &9.505211e-01,9.479895e-01,9.454961e-01,9.430393e-01,9.406174e-01,
     &9.382288e-01,9.358717e-01,9.335446e-01,9.312456e-01,9.289731e-01,
     &9.267255e-01,9.245010e-01,9.222980e-01,9.201147e-01,9.179496e-01,
     &9.158008e-01,9.136667e-01,9.115457e-01,9.094359e-01,9.073358e-01,
     &9.052436e-01,9.031577e-01,9.010763e-01,8.989977e-01,8.969203e-01,
     &8.948423e-01,8.927620e-01,8.906778e-01,8.885879e-01,8.864907e-01,
     &8.843843e-01/
      DATA (SSAICE3(I,21),I=1,46) /
C    BAND 21
     &9.934014e-01,9.899331e-01,9.865537e-01,9.832610e-01,9.800523e-01,
     &9.769254e-01,9.738777e-01,9.709069e-01,9.680106e-01,9.651862e-01,
     &9.624315e-01,9.597439e-01,9.571212e-01,9.545608e-01,9.520605e-01,
     &9.496177e-01,9.472301e-01,9.448954e-01,9.426111e-01,9.403749e-01,
     &9.381843e-01,9.360370e-01,9.339307e-01,9.318629e-01,9.298313e-01,
     &9.278336e-01,9.258673e-01,9.239302e-01,9.220198e-01,9.201338e-01,
     &9.182700e-01,9.164258e-01,9.145991e-01,9.127874e-01,9.109884e-01,
     &9.091999e-01,9.074194e-01,9.056447e-01,9.038735e-01,9.021033e-01,
     &9.003320e-01,8.985572e-01,8.967766e-01,8.949879e-01,8.931888e-01,
     &8.913770e-01/
      DATA (SSAICE3(I,22),I=1,46) /
C    BAND 22
     &9.994833e-01,9.992055e-01,9.989278e-01,9.986500e-01,9.983724e-01,
     &9.980947e-01,9.978172e-01,9.975397e-01,9.972623e-01,9.969849e-01,
     &9.967077e-01,9.964305e-01,9.961535e-01,9.958765e-01,9.955997e-01,
     &9.953230e-01,9.950464e-01,9.947699e-01,9.944936e-01,9.942174e-01,
     &9.939414e-01,9.936656e-01,9.933899e-01,9.931144e-01,9.928390e-01,
     &9.925639e-01,9.922889e-01,9.920141e-01,9.917396e-01,9.914652e-01,
     &9.911911e-01,9.909171e-01,9.906434e-01,9.903700e-01,9.900967e-01,
     &9.898237e-01,9.895510e-01,9.892784e-01,9.890062e-01,9.887342e-01,
     &9.884625e-01,9.881911e-01,9.879199e-01,9.876490e-01,9.873784e-01,
     &9.871081e-01/
      DATA (SSAICE3(I,23),I=1,46) /
C    BAND 23
     &9.999343e-01,9.998917e-01,9.998492e-01,9.998067e-01,9.997642e-01,
     &9.997218e-01,9.996795e-01,9.996372e-01,9.995949e-01,9.995528e-01,
     &9.995106e-01,9.994686e-01,9.994265e-01,9.993845e-01,9.993426e-01,
     &9.993007e-01,9.992589e-01,9.992171e-01,9.991754e-01,9.991337e-01,
     &9.990921e-01,9.990505e-01,9.990089e-01,9.989674e-01,9.989260e-01,
     &9.988846e-01,9.988432e-01,9.988019e-01,9.987606e-01,9.987194e-01,
     &9.986782e-01,9.986370e-01,9.985959e-01,9.985549e-01,9.985139e-01,
     &9.984729e-01,9.984319e-01,9.983910e-01,9.983502e-01,9.983094e-01,
     &9.982686e-01,9.982279e-01,9.981872e-01,9.981465e-01,9.981059e-01,
     &9.980653e-01/
      DATA (SSAICE3(I,24),I=1,46) /
C    BAND 24
     &9.999978e-01,9.999965e-01,9.999952e-01,9.999939e-01,9.999926e-01,
     &9.999913e-01,9.999900e-01,9.999887e-01,9.999873e-01,9.999860e-01,
     &9.999847e-01,9.999834e-01,9.999821e-01,9.999808e-01,9.999795e-01,
     &9.999782e-01,9.999769e-01,9.999756e-01,9.999743e-01,9.999730e-01,
     &9.999717e-01,9.999704e-01,9.999691e-01,9.999678e-01,9.999665e-01,
     &9.999652e-01,9.999639e-01,9.999626e-01,9.999613e-01,9.999600e-01,
     &9.999587e-01,9.999574e-01,9.999561e-01,9.999548e-01,9.999535e-01,
     &9.999522e-01,9.999509e-01,9.999496e-01,9.999483e-01,9.999470e-01,
     &9.999457e-01,9.999444e-01,9.999431e-01,9.999418e-01,9.999405e-01,
     &9.999392e-01/
      DATA (SSAICE3(I,25),I=1,46) /
C    BAND 25
     &9.999994e-01,9.999993e-01,9.999991e-01,9.999990e-01,9.999989e-01,
     &9.999987e-01,9.999986e-01,9.999984e-01,9.999983e-01,9.999982e-01,
     &9.999980e-01,9.999979e-01,9.999977e-01,9.999976e-01,9.999975e-01,
     &9.999973e-01,9.999972e-01,9.999970e-01,9.999969e-01,9.999967e-01,
     &9.999966e-01,9.999965e-01,9.999963e-01,9.999962e-01,9.999960e-01,
     &9.999959e-01,9.999957e-01,9.999956e-01,9.999954e-01,9.999953e-01,
     &9.999952e-01,9.999950e-01,9.999949e-01,9.999947e-01,9.999946e-01,
     &9.999944e-01,9.999943e-01,9.999941e-01,9.999940e-01,9.999939e-01,
     &9.999937e-01,9.999936e-01,9.999934e-01,9.999933e-01,9.999931e-01,
     &9.999930e-01/
      DATA (SSAICE3(I,26),I=1,46) /
C    BAND 26
     &9.999997e-01,9.999995e-01,9.999992e-01,9.999990e-01,9.999987e-01,
     &9.999985e-01,9.999983e-01,9.999980e-01,9.999978e-01,9.999976e-01,
     &9.999973e-01,9.999971e-01,9.999969e-01,9.999967e-01,9.999965e-01,
     &9.999963e-01,9.999960e-01,9.999958e-01,9.999956e-01,9.999954e-01,
     &9.999952e-01,9.999950e-01,9.999948e-01,9.999946e-01,9.999944e-01,
     &9.999942e-01,9.999939e-01,9.999937e-01,9.999935e-01,9.999933e-01,
     &9.999931e-01,9.999929e-01,9.999927e-01,9.999925e-01,9.999923e-01,
     &9.999920e-01,9.999918e-01,9.999916e-01,9.999914e-01,9.999911e-01,
     &9.999909e-01,9.999907e-01,9.999905e-01,9.999902e-01,9.999900e-01,
     &9.999897e-01/
      DATA (SSAICE3(I,27),I=1,46) /
C    BAND 27
     &9.999991e-01,9.999985e-01,9.999980e-01,9.999974e-01,9.999968e-01,
     &9.999963e-01,9.999957e-01,9.999951e-01,9.999946e-01,9.999940e-01,
     &9.999934e-01,9.999929e-01,9.999923e-01,9.999918e-01,9.999912e-01,
     &9.999907e-01,9.999901e-01,9.999896e-01,9.999891e-01,9.999885e-01,
     &9.999880e-01,9.999874e-01,9.999869e-01,9.999863e-01,9.999858e-01,
     &9.999853e-01,9.999847e-01,9.999842e-01,9.999836e-01,9.999831e-01,
     &9.999826e-01,9.999820e-01,9.999815e-01,9.999809e-01,9.999804e-01,
     &9.999798e-01,9.999793e-01,9.999787e-01,9.999782e-01,9.999776e-01,
     &9.999770e-01,9.999765e-01,9.999759e-01,9.999754e-01,9.999748e-01,
     &9.999742e-01/
      DATA (SSAICE3(I,28),I=1,46) /
C    BAND 28
     &9.999975e-01,9.999961e-01,9.999946e-01,9.999931e-01,9.999917e-01,
     &9.999903e-01,9.999888e-01,9.999874e-01,9.999859e-01,9.999845e-01,
     &9.999831e-01,9.999816e-01,9.999802e-01,9.999788e-01,9.999774e-01,
     &9.999759e-01,9.999745e-01,9.999731e-01,9.999717e-01,9.999702e-01,
     &9.999688e-01,9.999674e-01,9.999660e-01,9.999646e-01,9.999631e-01,
     &9.999617e-01,9.999603e-01,9.999589e-01,9.999574e-01,9.999560e-01,
     &9.999546e-01,9.999532e-01,9.999517e-01,9.999503e-01,9.999489e-01,
     &9.999474e-01,9.999460e-01,9.999446e-01,9.999431e-01,9.999417e-01,
     &9.999403e-01,9.999388e-01,9.999374e-01,9.999359e-01,9.999345e-01,
     &9.999330e-01/
      DATA (SSAICE3(I,29),I=1,46) /
C    BAND 29
     &4.526500e-01,5.287890e-01,5.410487e-01,5.459865e-01,5.485149e-01,
     &5.498914e-01,5.505895e-01,5.508310e-01,5.507364e-01,5.503793e-01,
     &5.498090e-01,5.490612e-01,5.481637e-01,5.471395e-01,5.460083e-01,
     &5.447878e-01,5.434946e-01,5.421442e-01,5.407514e-01,5.393309e-01,
     &5.378970e-01,5.364641e-01,5.350464e-01,5.336582e-01,5.323140e-01,
     &5.310283e-01,5.298158e-01,5.286914e-01,5.276704e-01,5.267680e-01,
     &5.260000e-01,5.253823e-01,5.249311e-01,5.246629e-01,5.245946e-01,
     &5.247434e-01,5.251268e-01,5.257626e-01,5.266693e-01,5.278653e-01,
     &5.293698e-01,5.312022e-01,5.333823e-01,5.359305e-01,5.388676e-01,
     &5.422146e-01/
C     ASYMMETRY FACTOR: Unitless
      DATA (ASYICE3(I,16),I=1,46) /
C    BAND 16
     &8.340752e-01,8.435170e-01,8.517487e-01,8.592064e-01,8.660387e-01,
     &8.723204e-01,8.780997e-01,8.834137e-01,8.882934e-01,8.927662e-01,
     &8.968577e-01,9.005914e-01,9.039899e-01,9.070745e-01,9.098659e-01,
     &9.123836e-01,9.146466e-01,9.166734e-01,9.184817e-01,9.200886e-01,
     &9.215109e-01,9.227648e-01,9.238661e-01,9.248304e-01,9.256727e-01,
     &9.264078e-01,9.270505e-01,9.276150e-01,9.281156e-01,9.285662e-01,
     &9.289806e-01,9.293726e-01,9.297557e-01,9.301435e-01,9.305491e-01,
     &9.309859e-01,9.314671e-01,9.320055e-01,9.326140e-01,9.333053e-01,
     &9.340919e-01,9.349861e-01,9.360000e-01,9.371451e-01,9.384329e-01,
     &9.398744e-01/
      DATA (ASYICE3(I,17),I=1,46) /
C    BAND 17
     &8.728160e-01,8.777333e-01,8.823754e-01,8.867535e-01,8.908785e-01,
     &8.947611e-01,8.984118e-01,9.018408e-01,9.050582e-01,9.080739e-01,
     &9.108976e-01,9.135388e-01,9.160068e-01,9.183106e-01,9.204595e-01,
     &9.224620e-01,9.243271e-01,9.260632e-01,9.276788e-01,9.291822e-01,
     &9.305817e-01,9.318853e-01,9.331012e-01,9.342372e-01,9.353013e-01,
     &9.363013e-01,9.372450e-01,9.381400e-01,9.389939e-01,9.398145e-01,
     &9.406092e-01,9.413856e-01,9.421511e-01,9.429131e-01,9.436790e-01,
     &9.444561e-01,9.452517e-01,9.460729e-01,9.469270e-01,9.478209e-01,
     &9.487617e-01,9.497562e-01,9.508112e-01,9.519335e-01,9.531294e-01,
     &9.544055e-01/
      DATA (ASYICE3(I,18),I=1,46) /
C    BAND 18
     &7.897566e-01,7.948704e-01,7.998041e-01,8.045623e-01,8.091495e-01,
     &8.135702e-01,8.178290e-01,8.219305e-01,8.258790e-01,8.296792e-01,
     &8.333355e-01,8.368524e-01,8.402343e-01,8.434856e-01,8.466108e-01,
     &8.496143e-01,8.525004e-01,8.552737e-01,8.579384e-01,8.604990e-01,
     &8.629597e-01,8.653250e-01,8.675992e-01,8.697867e-01,8.718916e-01,
     &8.739185e-01,8.758715e-01,8.777551e-01,8.795734e-01,8.813308e-01,
     &8.830315e-01,8.846799e-01,8.862802e-01,8.878366e-01,8.893534e-01,
     &8.908350e-01,8.922854e-01,8.937090e-01,8.951099e-01,8.964925e-01,
     &8.978609e-01,8.992192e-01,9.005718e-01,9.019229e-01,9.032765e-01,
     &9.046369e-01/
      DATA (ASYICE3(I,19),I=1,46) /
C    BAND 19
     &7.812615e-01,7.887764e-01,7.959664e-01,8.028413e-01,8.094109e-01,
     &8.156849e-01,8.216730e-01,8.273846e-01,8.328294e-01,8.380166e-01,
     &8.429556e-01,8.476556e-01,8.521258e-01,8.563753e-01,8.604131e-01,
     &8.642481e-01,8.678893e-01,8.713455e-01,8.746254e-01,8.777378e-01,
     &8.806914e-01,8.834948e-01,8.861566e-01,8.886854e-01,8.910897e-01,
     &8.933779e-01,8.955586e-01,8.976402e-01,8.996311e-01,9.015398e-01,
     &9.033745e-01,9.051436e-01,9.068555e-01,9.085185e-01,9.101410e-01,
     &9.117311e-01,9.132972e-01,9.148476e-01,9.163905e-01,9.179340e-01,
     &9.194864e-01,9.210559e-01,9.226505e-01,9.242784e-01,9.259476e-01,
     &9.276661e-01/
      DATA (ASYICE3(I,20),I=1,46) /
C    BAND 20
     &7.640720e-01,7.691119e-01,7.739941e-01,7.787222e-01,7.832998e-01,
     &7.877304e-01,7.920177e-01,7.961652e-01,8.001765e-01,8.040551e-01,
     &8.078044e-01,8.114280e-01,8.149294e-01,8.183119e-01,8.215791e-01,
     &8.247344e-01,8.277812e-01,8.307229e-01,8.335629e-01,8.363046e-01,
     &8.389514e-01,8.415067e-01,8.439738e-01,8.463560e-01,8.486568e-01,
     &8.508795e-01,8.530274e-01,8.551039e-01,8.571122e-01,8.590558e-01,
     &8.609378e-01,8.627618e-01,8.645309e-01,8.662485e-01,8.679178e-01,
     &8.695423e-01,8.711251e-01,8.726697e-01,8.741792e-01,8.756571e-01,
     &8.771065e-01,8.785307e-01,8.799331e-01,8.813169e-01,8.826854e-01,
     &8.840419e-01/
      DATA (ASYICE3(I,21),I=1,46) /
C    BAND 21
     &7.602598e-01,7.651572e-01,7.699014e-01,7.744962e-01,7.789452e-01,
     &7.832522e-01,7.874205e-01,7.914538e-01,7.953555e-01,7.991290e-01,
     &8.027777e-01,8.063049e-01,8.097140e-01,8.130081e-01,8.161906e-01,
     &8.192645e-01,8.222331e-01,8.250993e-01,8.278664e-01,8.305374e-01,
     &8.331153e-01,8.356030e-01,8.380037e-01,8.403201e-01,8.425553e-01,
     &8.447121e-01,8.467935e-01,8.488022e-01,8.507412e-01,8.526132e-01,
     &8.544210e-01,8.561675e-01,8.578554e-01,8.594875e-01,8.610665e-01,
     &8.625951e-01,8.640760e-01,8.655119e-01,8.669055e-01,8.682594e-01,
     &8.695763e-01,8.708587e-01,8.721094e-01,8.733308e-01,8.745255e-01,
     &8.756961e-01/
      DATA (ASYICE3(I,22),I=1,46) /
C    BAND 22
     &7.568957e-01,7.606995e-01,7.644072e-01,7.680204e-01,7.715402e-01,
     &7.749682e-01,7.783057e-01,7.815541e-01,7.847148e-01,7.877892e-01,
     &7.907786e-01,7.936846e-01,7.965084e-01,7.992515e-01,8.019153e-01,
     &8.045011e-01,8.070103e-01,8.094444e-01,8.118048e-01,8.140927e-01,
     &8.163097e-01,8.184571e-01,8.205364e-01,8.225488e-01,8.244958e-01,
     &8.263789e-01,8.281993e-01,8.299586e-01,8.316580e-01,8.332991e-01,
     &8.348831e-01,8.364115e-01,8.378857e-01,8.393071e-01,8.406770e-01,
     &8.419969e-01,8.432682e-01,8.444923e-01,8.456706e-01,8.468044e-01,
     &8.478952e-01,8.489444e-01,8.499533e-01,8.509234e-01,8.518561e-01,
     &8.527528e-01/
      DATA (ASYICE3(I,23),I=1,46) /
C    BAND 23
     &7.575066e-01,7.606912e-01,7.638236e-01,7.669035e-01,7.699306e-01,
     &7.729046e-01,7.758254e-01,7.786926e-01,7.815060e-01,7.842654e-01,
     &7.869705e-01,7.896211e-01,7.922168e-01,7.947574e-01,7.972428e-01,
     &7.996726e-01,8.020466e-01,8.043646e-01,8.066262e-01,8.088313e-01,
     &8.109796e-01,8.130709e-01,8.151049e-01,8.170814e-01,8.190001e-01,
     &8.208608e-01,8.226632e-01,8.244071e-01,8.260924e-01,8.277186e-01,
     &8.292856e-01,8.307932e-01,8.322411e-01,8.336291e-01,8.349570e-01,
     &8.362244e-01,8.374312e-01,8.385772e-01,8.396621e-01,8.406856e-01,
     &8.416476e-01,8.425479e-01,8.433861e-01,8.441620e-01,8.448755e-01,
     &8.455263e-01/
      DATA (ASYICE3(I,24),I=1,46) /
C    BAND 24
     &7.568829e-01,7.597947e-01,7.626745e-01,7.655212e-01,7.683337e-01,
     &7.711111e-01,7.738523e-01,7.765565e-01,7.792225e-01,7.818494e-01,
     &7.844362e-01,7.869819e-01,7.894854e-01,7.919459e-01,7.943623e-01,
     &7.967337e-01,7.990590e-01,8.013373e-01,8.035676e-01,8.057488e-01,
     &8.078802e-01,8.099605e-01,8.119890e-01,8.139645e-01,8.158862e-01,
     &8.177530e-01,8.195641e-01,8.213183e-01,8.230149e-01,8.246527e-01,
     &8.262308e-01,8.277483e-01,8.292042e-01,8.305976e-01,8.319275e-01,
     &8.331929e-01,8.343929e-01,8.355265e-01,8.365928e-01,8.375909e-01,
     &8.385197e-01,8.393784e-01,8.401659e-01,8.408815e-01,8.415240e-01,
     &8.420926e-01/
      DATA (ASYICE3(I,25),I=1,46) /
C    BAND 25
     &7.548616e-01,7.575454e-01,7.602153e-01,7.628696e-01,7.655067e-01,
     &7.681249e-01,7.707225e-01,7.732978e-01,7.758492e-01,7.783750e-01,
     &7.808735e-01,7.833430e-01,7.857819e-01,7.881886e-01,7.905612e-01,
     &7.928983e-01,7.951980e-01,7.974588e-01,7.996789e-01,8.018567e-01,
     &8.039905e-01,8.060787e-01,8.081196e-01,8.101115e-01,8.120527e-01,
     &8.139416e-01,8.157764e-01,8.175557e-01,8.192776e-01,8.209405e-01,
     &8.225427e-01,8.240826e-01,8.255585e-01,8.269688e-01,8.283117e-01,
     &8.295856e-01,8.307889e-01,8.319198e-01,8.329767e-01,8.339579e-01,
     &8.348619e-01,8.356868e-01,8.364311e-01,8.370930e-01,8.376710e-01,
     &8.381633e-01/
      DATA (ASYICE3(I,26),I=1,46) /
C    BAND 26
     &7.491854e-01,7.518523e-01,7.545089e-01,7.571534e-01,7.597839e-01,
     &7.623987e-01,7.649959e-01,7.675737e-01,7.701303e-01,7.726639e-01,
     &7.751727e-01,7.776548e-01,7.801084e-01,7.825318e-01,7.849230e-01,
     &7.872804e-01,7.896020e-01,7.918862e-01,7.941309e-01,7.963345e-01,
     &7.984951e-01,8.006109e-01,8.026802e-01,8.047009e-01,8.066715e-01,
     &8.085900e-01,8.104546e-01,8.122636e-01,8.140150e-01,8.157072e-01,
     &8.173382e-01,8.189063e-01,8.204096e-01,8.218464e-01,8.232148e-01,
     &8.245130e-01,8.257391e-01,8.268915e-01,8.279682e-01,8.289675e-01,
     &8.298875e-01,8.307264e-01,8.314824e-01,8.321537e-01,8.327385e-01,
     &8.332350e-01/
      DATA (ASYICE3(I,27),I=1,46) /
C    BAND 27
     &7.397086e-01,7.424069e-01,7.450955e-01,7.477725e-01,7.504362e-01,
     &7.530846e-01,7.557159e-01,7.583283e-01,7.609199e-01,7.634888e-01,
     &7.660332e-01,7.685512e-01,7.710411e-01,7.735009e-01,7.759288e-01,
     &7.783229e-01,7.806814e-01,7.830024e-01,7.852841e-01,7.875246e-01,
     &7.897221e-01,7.918748e-01,7.939807e-01,7.960380e-01,7.980449e-01,
     &7.999995e-01,8.019000e-01,8.037445e-01,8.055311e-01,8.072581e-01,
     &8.089235e-01,8.105255e-01,8.120623e-01,8.135319e-01,8.149326e-01,
     &8.162626e-01,8.175198e-01,8.187025e-01,8.198089e-01,8.208371e-01,
     &8.217852e-01,8.226514e-01,8.234338e-01,8.241306e-01,8.247399e-01,
     &8.252599e-01/
      DATA (ASYICE3(I,28),I=1,46) /
C    BAND 28
     &7.224533e-01,7.251681e-01,7.278728e-01,7.305654e-01,7.332444e-01,
     &7.359078e-01,7.385539e-01,7.411808e-01,7.437869e-01,7.463702e-01,
     &7.489291e-01,7.514616e-01,7.539661e-01,7.564408e-01,7.588837e-01,
     &7.612933e-01,7.636676e-01,7.660049e-01,7.683034e-01,7.705612e-01,
     &7.727767e-01,7.749480e-01,7.770733e-01,7.791509e-01,7.811789e-01,
     &7.831556e-01,7.850791e-01,7.869478e-01,7.887597e-01,7.905131e-01,
     &7.922062e-01,7.938372e-01,7.954044e-01,7.969059e-01,7.983399e-01,
     &7.997047e-01,8.009985e-01,8.022195e-01,8.033658e-01,8.044357e-01,
     &8.054275e-01,8.063392e-01,8.071692e-01,8.079157e-01,8.085768e-01,
     &8.091507e-01/
      DATA (ASYICE3(I,29),I=1,46) /
C    BAND 29
     &8.850026e-01,9.005489e-01,9.069242e-01,9.121799e-01,9.168987e-01,
     &9.212259e-01,9.252176e-01,9.289028e-01,9.323000e-01,9.354235e-01,
     &9.382858e-01,9.408985e-01,9.432734e-01,9.454218e-01,9.473557e-01,
     &9.490871e-01,9.506282e-01,9.519917e-01,9.531904e-01,9.542374e-01,
     &9.551461e-01,9.559298e-01,9.566023e-01,9.571775e-01,9.576692e-01,
     &9.580916e-01,9.584589e-01,9.587853e-01,9.590851e-01,9.593729e-01,
     &9.596632e-01,9.599705e-01,9.603096e-01,9.606954e-01,9.611427e-01,
     &9.616667e-01,9.622826e-01,9.630060e-01,9.638524e-01,9.648379e-01,
     &9.659788e-01,9.672916e-01,9.687933e-01,9.705014e-01,9.724337e-01,
     &9.746084e-01/
C     FDELTA: Unitless
      DATA (FDLICE3(I,16),I=1,46) /
C    BAND 16
     &4.959277e-02,4.685292e-02,4.426104e-02,4.181231e-02,3.950191e-02,
     &3.732500e-02,3.527675e-02,3.335235e-02,3.154697e-02,2.985578e-02,
     &2.827395e-02,2.679666e-02,2.541909e-02,2.413640e-02,2.294378e-02,
     &2.183639e-02,2.080940e-02,1.985801e-02,1.897736e-02,1.816265e-02,
     &1.740905e-02,1.671172e-02,1.606585e-02,1.546661e-02,1.490917e-02,
     &1.438870e-02,1.390038e-02,1.343939e-02,1.300089e-02,1.258006e-02,
     &1.217208e-02,1.177212e-02,1.137536e-02,1.097696e-02,1.057210e-02,
     &1.015596e-02,9.723704e-03,9.270516e-03,8.791565e-03,8.282026e-03,
     &7.737072e-03,7.151879e-03,6.521619e-03,5.841467e-03,5.106597e-03,
     &4.312183e-03/
      DATA (FDLICE3(I,17),I=1,46) /
C    BAND 17
     &5.071224e-02,5.000217e-02,4.933872e-02,4.871992e-02,4.814380e-02,
     &4.760839e-02,4.711170e-02,4.665177e-02,4.622662e-02,4.583426e-02,
     &4.547274e-02,4.514007e-02,4.483428e-02,4.455340e-02,4.429544e-02,
     &4.405844e-02,4.384041e-02,4.363939e-02,4.345340e-02,4.328047e-02,
     &4.311861e-02,4.296586e-02,4.282024e-02,4.267977e-02,4.254248e-02,
     &4.240640e-02,4.226955e-02,4.212995e-02,4.198564e-02,4.183462e-02,
     &4.167494e-02,4.150462e-02,4.132167e-02,4.112413e-02,4.091003e-02,
     &4.067737e-02,4.042420e-02,4.014854e-02,3.984840e-02,3.952183e-02,
     &3.916683e-02,3.878144e-02,3.836368e-02,3.791158e-02,3.742316e-02,
     &3.689645e-02/
      DATA (FDLICE3(I,18),I=1,46) /
C    BAND 18
     &1.062938e-01,1.065234e-01,1.067822e-01,1.070682e-01,1.073793e-01,
     &1.077137e-01,1.080693e-01,1.084442e-01,1.088364e-01,1.092439e-01,
     &1.096647e-01,1.100970e-01,1.105387e-01,1.109878e-01,1.114423e-01,
     &1.119004e-01,1.123599e-01,1.128190e-01,1.132757e-01,1.137279e-01,
     &1.141738e-01,1.146113e-01,1.150385e-01,1.154534e-01,1.158540e-01,
     &1.162383e-01,1.166045e-01,1.169504e-01,1.172741e-01,1.175738e-01,
     &1.178472e-01,1.180926e-01,1.183080e-01,1.184913e-01,1.186405e-01,
     &1.187538e-01,1.188291e-01,1.188645e-01,1.188580e-01,1.188076e-01,
     &1.187113e-01,1.185672e-01,1.183733e-01,1.181277e-01,1.178282e-01,
     &1.174731e-01/
      DATA (FDLICE3(I,19),I=1,46) /
C    BAND 19
     &1.076195e-01,1.065195e-01,1.054696e-01,1.044673e-01,1.035099e-01,
     &1.025951e-01,1.017203e-01,1.008831e-01,1.000808e-01,9.931116e-02,
     &9.857151e-02,9.785939e-02,9.717230e-02,9.650774e-02,9.586322e-02,
     &9.523623e-02,9.462427e-02,9.402484e-02,9.343544e-02,9.285358e-02,
     &9.227675e-02,9.170245e-02,9.112818e-02,9.055144e-02,8.996974e-02,
     &8.938056e-02,8.878142e-02,8.816981e-02,8.754323e-02,8.689919e-02,
     &8.623517e-02,8.554869e-02,8.483724e-02,8.409832e-02,8.332943e-02,
     &8.252807e-02,8.169175e-02,8.081795e-02,7.990419e-02,7.894796e-02,
     &7.794676e-02,7.689809e-02,7.579945e-02,7.464834e-02,7.344227e-02,
     &7.217872e-02/
      DATA (FDLICE3(I,20),I=1,46) /
C    BAND 20
     &1.119014e-01,1.122706e-01,1.126690e-01,1.130947e-01,1.135456e-01,
     &1.140199e-01,1.145154e-01,1.150302e-01,1.155623e-01,1.161096e-01,
     &1.166703e-01,1.172422e-01,1.178233e-01,1.184118e-01,1.190055e-01,
     &1.196025e-01,1.202008e-01,1.207983e-01,1.213931e-01,1.219832e-01,
     &1.225665e-01,1.231411e-01,1.237050e-01,1.242561e-01,1.247926e-01,
     &1.253122e-01,1.258132e-01,1.262934e-01,1.267509e-01,1.271836e-01,
     &1.275896e-01,1.279669e-01,1.283134e-01,1.286272e-01,1.289063e-01,
     &1.291486e-01,1.293522e-01,1.295150e-01,1.296351e-01,1.297104e-01,
     &1.297390e-01,1.297189e-01,1.296480e-01,1.295244e-01,1.293460e-01,
     &1.291109e-01/
      DATA (FDLICE3(I,21),I=1,46) /
C    BAND 21
     &1.133298e-01,1.136777e-01,1.140556e-01,1.144615e-01,1.148934e-01,
     &1.153492e-01,1.158269e-01,1.163243e-01,1.168396e-01,1.173706e-01,
     &1.179152e-01,1.184715e-01,1.190374e-01,1.196108e-01,1.201897e-01,
     &1.207720e-01,1.213558e-01,1.219389e-01,1.225194e-01,1.230951e-01,
     &1.236640e-01,1.242241e-01,1.247733e-01,1.253096e-01,1.258309e-01,
     &1.263352e-01,1.268205e-01,1.272847e-01,1.277257e-01,1.281415e-01,
     &1.285300e-01,1.288893e-01,1.292173e-01,1.295118e-01,1.297710e-01,
     &1.299927e-01,1.301748e-01,1.303154e-01,1.304124e-01,1.304637e-01,
     &1.304673e-01,1.304212e-01,1.303233e-01,1.301715e-01,1.299638e-01,
     &1.296983e-01/
      DATA (FDLICE3(I,22),I=1,46) /
C    BAND 22
     &1.145360e-01,1.153256e-01,1.161453e-01,1.169929e-01,1.178666e-01,
     &1.187641e-01,1.196835e-01,1.206227e-01,1.215796e-01,1.225522e-01,
     &1.235383e-01,1.245361e-01,1.255433e-01,1.265579e-01,1.275779e-01,
     &1.286011e-01,1.296257e-01,1.306494e-01,1.316703e-01,1.326862e-01,
     &1.336951e-01,1.346950e-01,1.356838e-01,1.366594e-01,1.376198e-01,
     &1.385629e-01,1.394866e-01,1.403889e-01,1.412678e-01,1.421212e-01,
     &1.429469e-01,1.437430e-01,1.445074e-01,1.452381e-01,1.459329e-01,
     &1.465899e-01,1.472069e-01,1.477819e-01,1.483128e-01,1.487976e-01,
     &1.492343e-01,1.496207e-01,1.499548e-01,1.502346e-01,1.504579e-01,
     &1.506227e-01/
      DATA (FDLICE3(I,23),I=1,46) /
C    BAND 23
     &1.153263e-01,1.161445e-01,1.169932e-01,1.178703e-01,1.187738e-01,
     &1.197016e-01,1.206516e-01,1.216217e-01,1.226099e-01,1.236141e-01,
     &1.246322e-01,1.256621e-01,1.267017e-01,1.277491e-01,1.288020e-01,
     &1.298584e-01,1.309163e-01,1.319736e-01,1.330281e-01,1.340778e-01,
     &1.351207e-01,1.361546e-01,1.371775e-01,1.381873e-01,1.391820e-01,
     &1.401593e-01,1.411174e-01,1.420540e-01,1.429671e-01,1.438547e-01,
     &1.447146e-01,1.455449e-01,1.463433e-01,1.471078e-01,1.478364e-01,
     &1.485270e-01,1.491774e-01,1.497857e-01,1.503497e-01,1.508674e-01,
     &1.513367e-01,1.517554e-01,1.521216e-01,1.524332e-01,1.526880e-01,
     &1.528840e-01/
      DATA (FDLICE3(I,24),I=1,46) /
C    BAND 24
     &1.160842e-01,1.169118e-01,1.177697e-01,1.186556e-01,1.195676e-01,
     &1.205036e-01,1.214616e-01,1.224394e-01,1.234349e-01,1.244463e-01,
     &1.254712e-01,1.265078e-01,1.275539e-01,1.286075e-01,1.296664e-01,
     &1.307287e-01,1.317923e-01,1.328550e-01,1.339149e-01,1.349699e-01,
     &1.360179e-01,1.370567e-01,1.380845e-01,1.390991e-01,1.400984e-01,
     &1.410803e-01,1.420429e-01,1.429840e-01,1.439016e-01,1.447936e-01,
     &1.456579e-01,1.464925e-01,1.472953e-01,1.480642e-01,1.487972e-01,
     &1.494923e-01,1.501472e-01,1.507601e-01,1.513287e-01,1.518511e-01,
     &1.523252e-01,1.527489e-01,1.531201e-01,1.534368e-01,1.536969e-01,
     &1.538984e-01/
      DATA (FDLICE3(I,25),I=1,46) /
C    BAND 25
     &1.168725e-01,1.177088e-01,1.185747e-01,1.194680e-01,1.203867e-01,
     &1.213288e-01,1.222923e-01,1.232750e-01,1.242750e-01,1.252903e-01,
     &1.263187e-01,1.273583e-01,1.284069e-01,1.294626e-01,1.305233e-01,
     &1.315870e-01,1.326517e-01,1.337152e-01,1.347756e-01,1.358308e-01,
     &1.368788e-01,1.379175e-01,1.389449e-01,1.399590e-01,1.409577e-01,
     &1.419389e-01,1.429007e-01,1.438410e-01,1.447577e-01,1.456488e-01,
     &1.465123e-01,1.473461e-01,1.481483e-01,1.489166e-01,1.496492e-01,
     &1.503439e-01,1.509988e-01,1.516118e-01,1.521808e-01,1.527038e-01,
     &1.531788e-01,1.536037e-01,1.539764e-01,1.542951e-01,1.545575e-01,
     &1.547617e-01/
      DATA (FDLICE3(I,26),I=1,46) /
C    BAND 26
     &1.180509e-01,1.189025e-01,1.197820e-01,1.206875e-01,1.216171e-01,
     &1.225687e-01,1.235404e-01,1.245303e-01,1.255363e-01,1.265564e-01,
     &1.275888e-01,1.286313e-01,1.296821e-01,1.307392e-01,1.318006e-01,
     &1.328643e-01,1.339284e-01,1.349908e-01,1.360497e-01,1.371029e-01,
     &1.381486e-01,1.391848e-01,1.402095e-01,1.412208e-01,1.422165e-01,
     &1.431949e-01,1.441539e-01,1.450915e-01,1.460058e-01,1.468947e-01,
     &1.477564e-01,1.485888e-01,1.493900e-01,1.501580e-01,1.508907e-01,
     &1.515864e-01,1.522428e-01,1.528582e-01,1.534305e-01,1.539578e-01,
     &1.544380e-01,1.548692e-01,1.552494e-01,1.555767e-01,1.558490e-01,
     &1.560645e-01/
      DATA (FDLICE3(I,27),I=1,46) /
C    BAND 27
     &1.200480e-01,1.209267e-01,1.218304e-01,1.227575e-01,1.237059e-01,
     &1.246739e-01,1.256595e-01,1.266610e-01,1.276765e-01,1.287041e-01,
     &1.297420e-01,1.307883e-01,1.318412e-01,1.328988e-01,1.339593e-01,
     &1.350207e-01,1.360813e-01,1.371393e-01,1.381926e-01,1.392396e-01,
     &1.402783e-01,1.413069e-01,1.423235e-01,1.433263e-01,1.443134e-01,
     &1.452830e-01,1.462332e-01,1.471622e-01,1.480681e-01,1.489490e-01,
     &1.498032e-01,1.506286e-01,1.514236e-01,1.521863e-01,1.529147e-01,
     &1.536070e-01,1.542614e-01,1.548761e-01,1.554491e-01,1.559787e-01,
     &1.564629e-01,1.568999e-01,1.572879e-01,1.576249e-01,1.579093e-01,
     &1.581390e-01/
      DATA (FDLICE3(I,28),I=1,46) /
C    BAND 28
     &1.247813e-01,1.256496e-01,1.265417e-01,1.274560e-01,1.283905e-01,
     &1.293436e-01,1.303135e-01,1.312983e-01,1.322964e-01,1.333060e-01,
     &1.343252e-01,1.353523e-01,1.363855e-01,1.374231e-01,1.384632e-01,
     &1.395042e-01,1.405441e-01,1.415813e-01,1.426140e-01,1.436404e-01,
     &1.446587e-01,1.456672e-01,1.466640e-01,1.476475e-01,1.486157e-01,
     &1.495671e-01,1.504997e-01,1.514117e-01,1.523016e-01,1.531673e-01,
     &1.540073e-01,1.548197e-01,1.556026e-01,1.563545e-01,1.570734e-01,
     &1.577576e-01,1.584054e-01,1.590149e-01,1.595843e-01,1.601120e-01,
     &1.605962e-01,1.610349e-01,1.614266e-01,1.617693e-01,1.620614e-01,
     &1.623011e-01/
      DATA (FDLICE3(I,29),I=1,46) /
C    BAND 29
     &1.006055e-01,9.549582e-02,9.063960e-02,8.602900e-02,8.165612e-02,
     &7.751308e-02,7.359199e-02,6.988496e-02,6.638412e-02,6.308156e-02,
     &5.996942e-02,5.703979e-02,5.428481e-02,5.169657e-02,4.926719e-02,
     &4.698880e-02,4.485349e-02,4.285339e-02,4.098061e-02,3.922727e-02,
     &3.758547e-02,3.604733e-02,3.460497e-02,3.325051e-02,3.197604e-02,
     &3.077369e-02,2.963558e-02,2.855381e-02,2.752050e-02,2.652776e-02,
     &2.556772e-02,2.463247e-02,2.371415e-02,2.280485e-02,2.189670e-02,
     &2.098180e-02,2.005228e-02,1.910024e-02,1.811781e-02,1.709709e-02,
     &1.603020e-02,1.490925e-02,1.372635e-02,1.247363e-02,1.114319e-02,
     &9.727157e-03/


      END



